/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(op)	{
	case 0x00: /* RLC B */
	{
		uint8 tmp = (cpu->bc.b.h << 1) | ((cpu->bc.b.h & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x01: /* RLC C */
	{
		uint8 tmp = (cpu->bc.b.l << 1) | ((cpu->bc.b.l & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x02: /* RLC D */
	{
		uint8 tmp = (cpu->de.b.h << 1) | ((cpu->de.b.h & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x03: /* RLC E */
	{
		uint8 tmp = (cpu->de.b.l << 1) | ((cpu->de.b.l & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x04: /* RLC H */
	{
		uint8 tmp = (cpu->hl.b.h << 1) | ((cpu->hl.b.h & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x05: /* RLC L */
	{
		uint8 tmp = (cpu->hl.b.l << 1) | ((cpu->hl.b.l & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x06: /* RLC (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m << 1) | ((m & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, m);
		cpu->cycles += 15;
		break;
	}
	case 0x07: /* RLC A */
	{
		uint8 tmp = (cpu->af.b.h << 1) | ((cpu->af.b.h & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(tmp & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x08: /* RRC B */
	{
		uint8 tmp = (cpu->bc.b.h >> 1) | ((cpu->bc.b.h & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.h & 0x01);
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x09: /* RRC C */
	{
		uint8 tmp = (cpu->bc.b.l >> 1) | ((cpu->bc.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.l & 0x01);
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x0A: /* RRC D */
	{
		uint8 tmp = (cpu->de.b.h >> 1) | ((cpu->de.b.h & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.h & 0x01);
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x0B: /* RRC E */
	{
		uint8 tmp = (cpu->de.b.l >> 1) | ((cpu->de.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.l & 0x01);
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x0C: /* RRC H */
	{
		uint8 tmp = (cpu->hl.b.h >> 1) | ((cpu->hl.b.h & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.h & 0x01);
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x0D: /* RRC L */
	{
		uint8 tmp = (cpu->hl.b.l >> 1) | ((cpu->hl.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.l & 0x01);
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x0E: /* RRC (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m >> 1) | ((m & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01);
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x0F: /* RRC A */
	{
		uint8 tmp = (cpu->af.b.h >> 1) | ((cpu->af.b.h & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->af.b.h & 0x01);
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x10: /* RL B */
	{
		uint8 tmp = (cpu->bc.b.h << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.h & 0x80) >> 7); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x11: /* RL C */
	{
		uint8 tmp = (cpu->bc.b.l << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.l & 0x80) >> 7); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x12: /* RL D */
	{
		uint8 tmp = (cpu->de.b.h << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.h & 0x80) >> 7); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x13: /* RL E */
	{
		uint8 tmp = (cpu->de.b.l << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.l & 0x80) >> 7); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x14: /* RL H */
	{
		uint8 tmp = (cpu->hl.b.h << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.h & 0x80) >> 7); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x15: /* RL L */
	{
		uint8 tmp = (cpu->hl.b.l << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.l & 0x80) >> 7); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x16: /* RL (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x17: /* RL A */
	{
		uint8 tmp = (cpu->af.b.h << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->af.b.h & 0x80) >> 7); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x18: /* RR B */
	{
		uint8 tmp = (cpu->bc.b.h >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.h & 0x01); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x19: /* RR C */
	{
		uint8 tmp = (cpu->bc.b.l >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.l & 0x01); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x1A: /* RR D */
	{
		uint8 tmp = (cpu->de.b.h >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.h & 0x01); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x1B: /* RR E */
	{
		uint8 tmp = (cpu->de.b.l >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.l & 0x01); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x1C: /* RR H */
	{
		uint8 tmp = (cpu->hl.b.h >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.h & 0x01); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x1D: /* RR L */
	{
		uint8 tmp = (cpu->hl.b.l >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.l & 0x01); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
		
	}
	case 0x1E: /* RR (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x1F: /* RR A */
	{
		uint8 tmp = (cpu->af.b.h >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->af.b.h & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x20: /* SLA B */
	{
		uint8 tmp = (cpu->bc.b.h << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.h & 0x80) >> 7); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x21: /* SLA C */
	{
		uint8 tmp = (cpu->bc.b.l << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.l & 0x80) >> 7); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x22: /* SLA D */
	{
		uint8 tmp = (cpu->de.b.h << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.h & 0x80) >> 7); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x23: /* SLA E */
	{
		uint8 tmp = (cpu->de.b.l << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.l & 0x80) >> 7); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x24: /* SLA H */
	{
		uint8 tmp = (cpu->hl.b.h << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.h & 0x80) >> 7); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x25: /* SLA L */
	{
		uint8 tmp = (cpu->hl.b.l << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.l & 0x80) >> 7); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x26: /* SLA (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x27: /* SLA A */
	{
		uint8 tmp = (cpu->af.b.h << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->af.b.h & 0x80) >> 7); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x28: /* SRA B */
	{
		uint8 tmp = (cpu->bc.b.h >> 1) | (cpu->bc.b.h & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.h & 0x01); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x29: /* SRA C */
	{
		uint8 tmp = (cpu->bc.b.l >> 1) | (cpu->bc.b.l & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.l & 0x01); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x2A: /* SRA D */
	{
		uint8 tmp = (cpu->de.b.h >> 1) | (cpu->de.b.h & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.h & 0x01); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x2B: /* SRA E */
	{
		uint8 tmp = (cpu->de.b.l >> 1) | (cpu->de.b.l & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.l & 0x01); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x2C: /* SRA H */
	{
		uint8 tmp = (cpu->hl.b.h >> 1) | (cpu->hl.b.h & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.h & 0x01); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x2D: /* SRA L */
	{
		uint8 tmp = (cpu->hl.b.l >> 1) | (cpu->hl.b.l & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.l & 0x01); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x2E: /* SRA (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m >> 1) | (m & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x2F: /* SRA A */
	{
		uint8 tmp = (cpu->af.b.h >> 1) | (cpu->af.b.h & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->af.b.h & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x30: /* SLL B */
	{
		uint8 tmp = (cpu->bc.b.h << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.h & 0x80) >> 7); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x31: /* SLL C */
	{
		uint8 tmp = (cpu->bc.b.l << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->bc.b.l & 0x80) >> 7); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x32: /* SLL D */
	{
		uint8 tmp = (cpu->de.b.h << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.h & 0x80) >> 7); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x33: /* SLL E */
	{
		uint8 tmp = (cpu->de.b.l << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->de.b.l & 0x80) >> 7); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x34: /* SLL H */
	{
		uint8 tmp = (cpu->hl.b.h << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.h & 0x80) >> 7); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x35: /* SLL L */
	{
		uint8 tmp = (cpu->hl.b.l << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->hl.b.l & 0x80) >> 7); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x36: /* SLL (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x37: /* SLL A */
	{
		uint8 tmp = (cpu->af.b.h << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((cpu->af.b.h & 0x80) >> 7); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x38: /* SRL B */
	{
		uint8 tmp = (cpu->bc.b.h >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.h & 0x01); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x39: /* SRL C */
	{
		uint8 tmp = (cpu->bc.b.l >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->bc.b.l & 0x01); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x3A: /* SRL D */
	{
		uint8 tmp = (cpu->de.b.h >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.h & 0x01); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x3B: /* SRL E */
	{
		uint8 tmp = (cpu->de.b.l >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->de.b.l & 0x01); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x3C: /* SRL H */
	{
		uint8 tmp = (cpu->hl.b.h >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.h & 0x01); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x3D: /* SRL L */
	{
		uint8 tmp = (cpu->hl.b.l >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->hl.b.l & 0x01); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x3E: /* SRL (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint8 tmp = (m >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 15;
		break;
	}
	case 0x3F: /* SRL A */
	{
		uint8 tmp = (cpu->af.b.h >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(cpu->af.b.h & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x40: /* BIT 0, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x41: /* BIT 0, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x42: /* BIT 0, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x43: /* BIT 0, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x44: /* BIT 0, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x45: /* BIT 0, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x46: /* BIT 0, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x47: /* BIT 0, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x48: /* BIT 1, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x49: /* BIT 1, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x4A: /* BIT 1, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x4B: /* BIT 1, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x4C: /* BIT 1, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x4D: /* BIT 1, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x4E: /* BIT 1, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x4F: /* BIT 1, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x50: /* BIT 2, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x51: /* BIT 2, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x52: /* BIT 2, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x53: /* BIT 2, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x54: /* BIT 2, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x55: /* BIT 2, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x56: /* BIT 2, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x57: /* BIT 2, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x58: /* BIT 3, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x59: /* BIT 3, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x5A: /* BIT 3, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x5B: /* BIT 3, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x5C: /* BIT 3, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x5D: /* BIT 3, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x5E: /* BIT 3, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x5F: /* BIT 3, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x60: /* BIT 4, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x61: /* BIT 4, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x62: /* BIT 4, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x63: /* BIT 4, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x64: /* BIT 4, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x65: /* BIT 4, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x66: /* BIT 4, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x67: /* BIT 4, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x68: /* BIT 5, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x69: /* BIT 5, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x6A: /* BIT 5, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x6B: /* BIT 5, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x6C: /* BIT 5, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x6D: /* BIT 5, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x6E: /* BIT 5, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x6F: /* BIT 5, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x70: /* BIT 6, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x71: /* BIT 6, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x72: /* BIT 6, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x73: /* BIT 6, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x74: /* BIT 6, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x75: /* BIT 6, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x76: /* BIT 6, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x77: /* BIT 6, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x78: /* BIT 7, B */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.h & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x79: /* BIT 7, C */
	{
		cpu->af.b.l = ZSPtable[cpu->bc.b.l & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x7A: /* BIT 7, D */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.h & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x7B: /* BIT 7, E */
	{
		cpu->af.b.l = ZSPtable[cpu->de.b.l & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x7C: /* BIT 7, H */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.h & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x7D: /* BIT 7, L */
	{
		cpu->af.b.l = ZSPtable[cpu->hl.b.l & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x7E: /* BIT 7, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->af.b.l = ZSPtable[m & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 12;
		break;
	}
	case 0x7F: /* BIT 7, A */
	{
		cpu->af.b.l = ZSPtable[cpu->af.b.h & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 8;
		break;
	}
	case 0x80: /* RES 0, B */
	{
		cpu->bc.b.h &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x81: /* RES 0, C */
	{
		cpu->bc.b.l &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x82: /* RES 0, D */
	{
		cpu->de.b.h &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x83: /* RES 0, E */
	{
		cpu->de.b.l &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x84: /* RES 0, H */
	{
		cpu->hl.b.h &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x85: /* RES 0, L */
	{
		cpu->hl.b.l &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x86: /* RES 0, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xFE);
		cpu->cycles += 15;
		break;
	}
	case 0x87: /* RES 0, A */
	{
		cpu->af.b.h &= 0xFE;
		cpu->cycles += 8;
		break;
	}
	case 0x88: /* RES 1, B */
	{
		cpu->bc.b.h &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x89: /* RES 1, C */
	{
		cpu->bc.b.l &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x8A: /* RES 1, D */
	{
		cpu->de.b.h &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x8B: /* RES 1, E */
	{
		cpu->de.b.l &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x8C: /* RES 1, H */
	{
		cpu->hl.b.h &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x8D: /* RES 1, L */
	{
		cpu->hl.b.l &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x8E: /* RES 1, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xFD);
		cpu->cycles += 15;
		break;
	}
	case 0x8F: /* RES 1, A */
	{
		cpu->af.b.h &= 0xFD;
		cpu->cycles += 8;
		break;
	}
	case 0x90: /* RES 2, B */
	{
		cpu->bc.b.h &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x91: /* RES 2, C */
	{
		cpu->bc.b.l &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x92: /* RES 2, D */
	{
		cpu->de.b.h &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x93: /* RES 2, E */
	{
		cpu->de.b.l &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x94: /* RES 2, H */
	{
		cpu->hl.b.h &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x95: /* RES 2, L */
	{
		cpu->hl.b.l &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x96: /* RES 2, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xFB);
		cpu->cycles += 15;
		break;
	}
	case 0x97: /* RES 2, A */
	{
		cpu->af.b.h &= 0xFB;
		cpu->cycles += 8;
		break;
	}
	case 0x98: /* RES 3, B */
	{
		cpu->bc.b.h &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x99: /* RES 3, C */
	{
		cpu->bc.b.l &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x9A: /* RES 3, D */
	{
		cpu->de.b.h &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x9B: /* RES 3, E */
	{
		cpu->de.b.l &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x9C: /* RES 3, H */
	{
		cpu->hl.b.h &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x9D: /* RES 3, L */
	{
		cpu->hl.b.l &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0x9E: /* RES 3, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xF7);
		cpu->cycles += 15;
		break;
	}
	case 0x9F: /* RES 3, A */
	{
		cpu->af.b.h &= 0xF7;
		cpu->cycles += 8;
		break;
	}
	case 0xA0: /* RES 4, B */
	{
		cpu->bc.b.h &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA1: /* RES 4, C */
	{
		cpu->bc.b.l &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA2: /* RES 4, D */
	{
		cpu->de.b.h &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA3: /* RES 4, E */
	{
		cpu->de.b.l &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA4: /* RES 4, H */
	{
		cpu->hl.b.h &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA5: /* RES 4, L */
	{
		cpu->hl.b.l &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA6: /* RES 4, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xEF);
		cpu->cycles += 15;
		break;
	}
	case 0xA7: /* RES 4, A */
	{
		cpu->af.b.h &= 0xEF;
		cpu->cycles += 8;
		break;
	}
	case 0xA8: /* RES 5, B */
	{
		cpu->bc.b.h &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xA9: /* RES 5, C */
	{
		cpu->bc.b.l &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xAA: /* RES 5, D */
	{
		cpu->de.b.h &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xAB: /* RES 5, E */
	{
		cpu->de.b.l &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xAC: /* RES 5, H */
	{
		cpu->hl.b.h &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xAD: /* RES 5, L */
	{
		cpu->hl.b.l &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xAE: /* RES 5, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xDF);
		cpu->cycles += 15;
		break;
	}
	case 0xAF: /* RES 5, A */
	{
		cpu->af.b.h &= 0xDF;
		cpu->cycles += 8;
		break;
	}
	case 0xB0: /* RES 6, B */
	{
		cpu->bc.b.h &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB1: /* RES 6, C */
	{
		cpu->bc.b.l &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB2: /* RES 6, D */
	{
		cpu->de.b.h &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB3: /* RES 6, E */
	{
		cpu->de.b.l &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB4: /* RES 6, H */
	{
		cpu->hl.b.h &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB5: /* RES 6, L */
	{
		cpu->hl.b.l &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB6: /* RES 6, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0xBF);
		cpu->cycles += 15;
		break;
	}
	case 0xB7: /* RES 6, A */
	{
		cpu->af.b.h &= 0xBF;
		cpu->cycles += 8;
		break;
	}
	case 0xB8: /* RES 7, B */
	{
		cpu->bc.b.h &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xB9: /* RES 7, C */
	{
		cpu->bc.b.l &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xBA: /* RES 7, D */
	{
		cpu->de.b.h &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xBB: /* RES 7, E */
	{
		cpu->de.b.l &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xBC: /* RES 7, H */
	{
		cpu->hl.b.h &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xBD: /* RES 7, L */
	{
		cpu->hl.b.l &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xBE: /* RES 7, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m & 0x7F);
		cpu->cycles += 15;
		break;
	}
	case 0xBF: /* RES 7, A */
	{
		cpu->af.b.h &= 0x7F;
		cpu->cycles += 8;
		break;
	}
	case 0xC0: /* SET 0, B */
	{
		cpu->bc.b.h |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC1: /* SET 0, C */
	{
		cpu->bc.b.l |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC2: /* SET 0, D */
	{
		cpu->de.b.h |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC3: /* SET 0, E */
	{
		cpu->de.b.l |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC4: /* SET 0, H */
	{
		cpu->hl.b.h |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC5: /* SET 0, L */
	{
		cpu->hl.b.l |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC6: /* SET 0, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x01);
		cpu->cycles += 15;
		break;
	}
	case 0xC7: /* SET 0, A */
	{
		cpu->af.b.h |= 0x01;
		cpu->cycles += 8;
		break;
	}
	case 0xC8: /* SET 1, B */
	{
		cpu->bc.b.h |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xC9: /* SET 1, C */
	{
		cpu->bc.b.l |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xCA: /* SET 1, D */
	{
		cpu->de.b.h |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xCB: /* SET 1, E */
	{
		cpu->de.b.l |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xCC: /* SET 1, H */
	{
		cpu->hl.b.h |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xCD: /* SET 1, L */
	{
		cpu->hl.b.l |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xCE: /* SET 1, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x02);
		cpu->cycles += 15;
		break;
	}
	case 0xCF: /* SET 1, A */
	{
		cpu->af.b.h |= 0x02;
		cpu->cycles += 8;
		break;
	}
	case 0xD0: /* SET 2, B */
	{
		cpu->bc.b.h |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD1: /* SET 2, C */
	{
		cpu->bc.b.l |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD2: /* SET 2, D */
	{
		cpu->de.b.h |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD3: /* SET 2, E */
	{
		cpu->de.b.l |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD4: /* SET 2, H */
	{
		cpu->hl.b.h |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD5: /* SET 2, L */
	{
		cpu->hl.b.l |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD6: /* SET 2, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x04);
		cpu->cycles += 15;
		break;
	}
	case 0xD7: /* SET 2, A */
	{
		cpu->af.b.h |= 0x04;
		cpu->cycles += 8;
		break;
	}
	case 0xD8: /* SET 3, B */
	{
		cpu->bc.b.h |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xD9: /* SET 3, C */
	{
		cpu->bc.b.l |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xDA: /* SET 3, D */
	{
		cpu->de.b.h |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xDB: /* SET 3, E */
	{
		cpu->de.b.l |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xDC: /* SET 3, H */
	{
		cpu->hl.b.h |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xDD: /* SET 3, L */
	{
		cpu->hl.b.l |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xDE: /* SET 3, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x08);
		cpu->cycles += 15;
		break;
	}
	case 0xDF: /* SET 3, A */
	{
		cpu->af.b.h |= 0x08;
		cpu->cycles += 8;
		break;
	}
	case 0xE0: /* SET 4, B */
	{
		cpu->bc.b.h |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE1: /* SET 4, C */
	{
		cpu->bc.b.l |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE2: /* SET 4, D */
	{
		cpu->de.b.h |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE3: /* SET 4, E */
	{
		cpu->de.b.l |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE4: /* SET 4, H */
	{
		cpu->hl.b.h |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE5: /* SET 4, L */
	{
		cpu->hl.b.l |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE6: /* SET 4, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x10);
		cpu->cycles += 15;
		break;
	}
	case 0xE7: /* SET 4, A */
	{
		cpu->af.b.h |= 0x10;
		cpu->cycles += 8;
		break;
	}
	case 0xE8: /* SET 5, B */
	{
		cpu->bc.b.h |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xE9: /* SET 5, C */
	{
		cpu->bc.b.l |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xEA: /* SET 5, D */
	{
		cpu->de.b.h |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xEB: /* SET 5, E */
	{
		cpu->de.b.l |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xEC: /* SET 5, H */
	{
		cpu->hl.b.h |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xED: /* SET 5, L */
	{
		cpu->hl.b.l |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xEE: /* SET 5, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x20);
		cpu->cycles += 15;
		break;
	}
	case 0xEF: /* SET 5, A */
	{
		cpu->af.b.h |= 0x20;
		cpu->cycles += 8;
		break;
	}
	case 0xF0: /* SET 6, B */
	{
		cpu->bc.b.h |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF1: /* SET 6, C */
	{
		cpu->bc.b.l |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF2: /* SET 6, D */
	{
		cpu->de.b.h |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF3: /* SET 6, E */
	{
		cpu->de.b.l |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF4: /* SET 6, H */
	{
		cpu->hl.b.h |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF5: /* SET 6, L */
	{
		cpu->hl.b.l |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF6: /* SET 6, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x40);
		cpu->cycles += 15;
		break;
	}
	case 0xF7: /* SET 6, A */
	{
		cpu->af.b.h |= 0x40;
		cpu->cycles += 8;
		break;
	}
	case 0xF8: /* SET 7, B */
	{
		cpu->bc.b.h |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xF9: /* SET 7, C */
	{
		cpu->bc.b.l |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xFA: /* SET 7, D */
	{
		cpu->de.b.h |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xFB: /* SET 7, E */
	{
		cpu->de.b.l |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xFC: /* SET 7, H */
	{
		cpu->hl.b.h |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xFD: /* SET 7, L */
	{
		cpu->hl.b.l |= 0x80;
		cpu->cycles += 8;
		break;
	}
	case 0xFE: /* SET 7, (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		cpu->mwrite(cpu->hl.w, m | 0x80);
		cpu->cycles += 15;
		break;
	}
	case 0xFF: /* SET 7, A */
	{
		cpu->af.b.h |= 0x80;
		cpu->cycles += 8;
		break;
	}
	default:
	{
		printf("Error in emulation, bailing out, opcode = 0xCB%02X\n", op);
		assert(0);
	}
}
