/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* This file contains the z80 core debugger that I wrote
   to compare how CrabZ80 worked versus the Z80 core from
   MAME. The code isn't pretty, but its here, in case
   anyone wants to compile their own binary with it.
   (These binaries cannot be released due to licensing
   restrictions on the MAME core that contradict the GPL).
*/

#include <stdio.h>
#include <stdlib.h>
#include "smsz80.h"

typedef struct	{
	uint16 addr;
	uint8 val;
} debug_memory_ent;

z80_t *crabz80;
extern Z80_Regs Z80;

debug_memory_ent crabmem[10];
debug_memory_ent mz80mem[10];

debug_memory_ent portread[10];
debug_memory_ent memread[200];

int debug_crab_ents = 0;
int debug_mz80_ents = 0;
int debug_port_reads1 = 0;
int debug_port_reads2 = 0;
int debug_mem_reads1 = 0;
int debug_mem_reads2 = 0;

static void sms_debug_memwrite(uint16 addr, uint8 val)	{
	crabmem[debug_crab_ents].addr = addr;
	crabmem[debug_crab_ents++].val = val;

	sms_mem_write(addr, val);
}

static uint8 sms_debug_portread(uint16 port)	{
	uint8 res = sms_port_read(port);

	portread[debug_port_reads1].addr = port;
	portread[debug_port_reads1++].val = res;

	return res;
}

static uint8 sms_debug_memread(uint16 addr)	{
	uint8 res = sms_mem_read(addr);
	
	memread[debug_mem_reads1].addr = addr;
	memread[debug_mem_reads1++].val = res;
	
	return res;
}

/* Glue code for mz80 */
void program_write_byte_8(UINT32 addr, UINT8 val)	{
	mz80mem[debug_mz80_ents].addr = addr;
	mz80mem[debug_mz80_ents].val = val;
	debug_mz80_ents++;
}

UINT8 program_read_byte_8(UINT32 addr)	{
	int i;
	//return memread[debug_mem_reads2++].val;
	for(i = 0; i < debug_mem_reads1; ++i)	{
		if(memread[i].addr == addr)	
			return memread[i].val;
	}

	return sms_mem_read(addr);
}

void io_write_byte_8(UINT32 port, UINT8 val)	{
}

UINT8 io_read_byte_8(UINT32 port)	{
	return portread[debug_port_reads2++].val;
}


static int _compare_registers()	{
	int f = 0;

	if(crabz80->af.b.h != Z80.af.b.h)	{
		printf("A does not match, %x v %x\n", crabz80->af.b.h, Z80.af.b.h);
		f++;
	}
	if(crabz80->af.b.l != (Z80.af.b.l & 0xD7))	{
		printf("F does not match, %02X v %02X\n", crabz80->af.b.l, Z80.af.b.l);
		f++;
	}
	if(crabz80->bc.b.h != Z80.bc.b.h)	{
		printf("B does not match, %x v %x\n", crabz80->bc.b.h, Z80.bc.b.h);
		f++;
	}
	if(crabz80->bc.b.l != Z80.bc.b.l)	{
		printf("C does not match, %x v %x\n", crabz80->bc.b.l, Z80.bc.b.l);
		f++;
	}
	if(crabz80->de.b.h != Z80.de.b.h)	{
		printf("D does not match, %x v %x\n", crabz80->de.b.h, Z80.de.b.h);
		f++;
	}
	if(crabz80->de.b.l != Z80.de.b.l)	{
		printf("E does not match, %x v %x\n", crabz80->de.b.l, Z80.de.b.l);
		f++;
	}
	if(crabz80->hl.b.h != Z80.hl.b.h)	{
		printf("H does not match, %x v %x\n", crabz80->hl.b.h, Z80.hl.b.h);
		f++;
	}
	if(crabz80->hl.b.l != Z80.hl.b.l)	{
		printf("L does not match, %x v %x\n", crabz80->hl.b.l, Z80.hl.b.l);
		f++;
	}
	if(crabz80->afp.b.h != Z80.af2.b.h)	{
		printf("Ap does not match, %x v %x\n", crabz80->afp.b.h, Z80.af2.b.h);
		f++;
	}
	if(crabz80->afp.b.l != (Z80.af2.b.l & 0xD7))	{
		printf("Fp does not match, %x v %x\n", crabz80->afp.b.l, Z80.af2.b.l);
		f++;
	}
	if(crabz80->bcp.b.h != Z80.bc2.b.h)	{
		printf("Bp does not match, %x v %x\n", crabz80->bcp.b.h, Z80.bc2.b.h);
		f++;
	}
	if(crabz80->bcp.b.l != Z80.bc2.b.l)	{
		printf("Cp does not match, %x v %x\n", crabz80->bcp.b.l, Z80.bc2.b.l);
		f++;
	}
	if(crabz80->dep.b.h != Z80.de2.b.h)	{
		printf("Dp does not match, %x v %x\n", crabz80->dep.b.h, Z80.de2.b.h);
		f++;
	}
	if(crabz80->dep.b.l != Z80.de2.b.l)	{
		printf("Ep does not match, %x v %x\n", crabz80->dep.b.l, Z80.de2.b.l);
		f++;
	}
	if(crabz80->hlp.b.h != Z80.hl2.b.h)	{
		printf("Hp does not match, %x v %x\n", crabz80->hlp.b.h, Z80.hl2.b.h);
		f++;
	}
	if(crabz80->hlp.b.l != Z80.hl2.b.l)	{
		printf("Lp does not match, %x v %x\n", crabz80->hlp.b.l, Z80.hl2.b.l);
		f++;
	}
	if(crabz80->pc.w != Z80.pc.w.l)	{
		printf("PC does not match, %x v %x\n", crabz80->pc.w, Z80.pc.w.l);
		f++;
	}
	if(crabz80->ix.w != Z80.ix.w.l)	{
		printf("IX does not match %x v %x\n", crabz80->ix.w, Z80.iy.w.l);
		f++;
	}
	if(crabz80->iy.w != Z80.iy.w.l)	{
		printf("IY does not match %x v %x\n", crabz80->iy.w, Z80.ix.w.l);
		f++;
	}
	if(crabz80->sp.w != Z80.sp.w.l)	{
		printf("SP does not match %x v %x\n", crabz80->sp.w, Z80.sp.w.l);
		f++;
	}
	if(crabz80->ir.b.h != Z80.i)	{
		printf("I does not match %x v %x\n", crabz80->ir.b.h, Z80.i);
		f++;
	}
	if(crabz80->ir.b.l != Z80.r)	{
		printf("R does not match %x v %x\n", crabz80->ir.b.l, Z80.r);
		f++;
	}

	return f;
}

static int _compare_memory()	{
	int i;
	int f = 0;

	for(i = 0; i < debug_crab_ents; ++i)	{
		if(crabmem[i].addr != mz80mem[i].addr)	{
			printf("Mismatched addresses! %02x v %02x\n", crabmem[i].addr, mz80mem[i].addr);
			f++;
		}
		else if(crabmem[i].val != mz80mem[i].val)	{
			printf("mismatched bytes! %02x v %02x\n", crabmem[i].val, mz80mem[i].val);
			f++;
		}
	}

	return f;
}

int sms_z80_init()	{
	crabz80 = (z80_t *)malloc(sizeof(z80_t));

	if(crabz80 == NULL)	{
		fprintf(stderr, "Out of memory while initializing Z80 in debug mode\n");
		return -1;
	}

	/* Initialize CrabZ80 */
	z80_init(crabz80);
	z80_reset(crabz80);

	z80_set_memwrite(crabz80, sms_debug_memwrite);
	z80_set_memread(crabz80, sms_debug_memread);
	z80_set_portwrite(crabz80, sms_port_write);
	z80_set_portread(crabz80, sms_debug_portread);

	_z80_init();

	return 0;
}

int sms_z80_shutdown()	{
	free(crabz80);
	_z80_exit();

	return 0;
}

void sms_z80_reset()	{
	z80_reset(crabz80);
	_z80_reset(NULL);
}

void sms_z80_irq()	{
	z80_pulse_irq(crabz80);
	_z80_set_irq_line(INPUT_LINE_NMI + 1, 1);
	_z80_set_irq_line(INPUT_LINE_NMI + 1, 0);
}

void sms_z80_nmi()	{
	z80_pulse_nmi(crabz80);
	_z80_set_irq_line(INPUT_LINE_NMI, 1);
	_z80_set_irq_line(INPUT_LINE_NMI, 0);
}

void sms_z80_run(int cycles)	{
	int cyclesdone = 0;
	unsigned int opcode;
	uint16 pc;
	int tmp;

	while(cyclesdone < cycles)	{
		if(crabz80->cycles)	{
			cyclesdone += crabz80->cycles;
			crabz80->cycles = 0;
			continue;
		}

		debug_crab_ents = debug_mz80_ents = 0;
		debug_port_reads1 = debug_port_reads2 = 0;
		debug_mem_reads1 = debug_mem_reads2 = 0;

		pc = crabz80->pc.w;

		opcode = sms_mem_read(crabz80->pc.w);

		if(opcode == 0xED || opcode == 0xCB || opcode == 0xDD || opcode == 0xFD)	{
			opcode |= (sms_mem_read(crabz80->pc.w + 1) << 8);
		}
		if((opcode & 0xFF00) == 0xCB00)	{
			opcode |= (sms_mem_read(crabz80->pc.w + 2) << 16) | (sms_mem_read(crabz80->pc.w + 3) << 24);
		}

		cyclesdone += z80_execute(crabz80, 1);
		_z80_execute(1);

		tmp = _compare_registers() + _compare_memory();

		if(tmp)	{
			printf("%d errors at: PC = 0x%04X, Opcode = 0x%02X", tmp, pc, opcode & 0xFF);
			switch(opcode & 0xFF)	{
				case 0xED:
				case 0xCB:
					printf("%02X\n", (opcode & 0xFF00) >> 8);
					break;
				case 0xDD:
				case 0xFD:
					if((opcode & 0xFF00) == 0xCB00)
						printf("%02X%02X%02X\n", (opcode & 0xFF00) >> 8, (opcode & 0xFF0000) >> 16,  (opcode & 0xFF000000) >> 24);
					else
						printf("%02X\n", (opcode & 0xFF00) >> 8);
					break;
				default:
					printf("\n");
			}
		}
	}
}
