/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef Z80_H
#define Z80_H

#include "z80_types.h"

typedef union	{
	struct	{
#ifdef __BIG_ENDIAN__
		uint8 h;
		uint8 l;
#else
		uint8 l;
		uint8 h;
#endif
	} b;
	uint16 w;
} reg_pair;

typedef struct z80_struct	{
	reg_pair af;
	reg_pair bc;
	reg_pair de;
	reg_pair hl;
	reg_pair ix;
	reg_pair iy;
	reg_pair pc;
	reg_pair sp;
	reg_pair ir;
	reg_pair afp;
	reg_pair bcp;
	reg_pair dep;
	reg_pair hlp;

	uint8 iff1;
	uint8 iff2;
	uint8 im;

	uint8 halt;
	uint8 ei;
	uint8 r_top;

	uint32 cycles;

	uint8 (*pread)(uint16 port);
	uint8 (*mread)(uint16 addr);

	void (*pwrite)(uint16 port, uint8 data);
	void (*mwrite)(uint16 addr, uint8 data);
} z80_t;

/* Flag definitions */
#define CF		0
#define NF		1
#define PF		2
#define XF		3
#define HF		4
#define YF		5
#define ZF		6
#define SF		7

/* Flag setting macros */
#define SET_FLAG(z80, n)	(z80)->af.b.l |= (1 << (n))
#define CLEAR_FLAG(z80, n)	(z80)->af.b.l &= (~(1 << (n)))
#define GET_FLAG(z80, n)	((z80)->af.b.l >> (n)) & 1

/* Function definitions */
void z80_init(z80_t *cpu);
void z80_reset(z80_t *cpu);

void z80_pulse_nmi(z80_t *cpu);
void z80_pulse_irq(z80_t *cpu);

uint32 z80_execute(z80_t *cpu, uint32 cycles);

void z80_set_portread(z80_t *cpu, uint8 (*pread)(uint16 port));
void z80_set_memread(z80_t *cpu, uint8 (*mread)(uint16 addr));
void z80_set_portwrite(z80_t *cpu, void (*pwrite)(uint16 port, uint8 data));
void z80_set_memwrite(z80_t *cpu, void (*mwrite)(uint16 addr, uint8 data));

#endif /* !Z80_H */
