/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(op2)	{
	case 0x06: /* RLC (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m << 1) | ((m & 0x80) >> 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x0E: /* RRC (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m >> 1) | ((m & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x16: /* RL (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x1E: /* RR (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x26: /* SLA (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m << 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x2E: /* SRA (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m >> 1) | (m & 0x80);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
    case 0x36: /* SLL (IX+d) */
    {
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m << 1) | 0x01;
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			((m & 0x80) >> 7); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
#endif
	case 0x3E: /* SRL (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		uint8 tmp = (m >> 1);
		cpu->af.b.l = ZSPtable[tmp] | /* S, Z, P */
			(m & 0x01); /* C */
		cpu->mwrite(cpu->ix.w + d, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0x46: /* BIT 0, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x01] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x4E: /* BIT 1, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x02] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x56: /* BIT 2, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x04] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x5E: /* BIT 3, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x08] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x66: /* BIT 4, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x10] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x6E: /* BIT 5, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x20] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x76: /* BIT 6, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x40] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x7E: /* BIT 7, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->af.b.l = ZSPtable[m & 0x80] | /* S, Z, P */
			0x10 | /* H */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 20;
		break;
	}
	case 0x86: /* RES 0, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xFE);
		cpu->cycles += 23;
		break;
	}
	case 0x8E: /* RES 1, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xFD);
		cpu->cycles += 23;
		break;
	}
	case 0x96: /* RES 2, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xFB);
		cpu->cycles += 23;
		break;
	}
	case 0x9E: /* RES 3, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xF7);
		cpu->cycles += 23;
		break;
	}
	case 0xA6: /* RES 4, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xEF);
		cpu->cycles += 23;
		break;
	}
	case 0xAE: /* RES 5, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xDF);
		cpu->cycles += 23;
		break;
	}
	case 0xB6: /* RES 6, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0xBF);
		cpu->cycles += 23;
		break;
	}
	case 0xBE: /* RES 7, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m & 0x7F);
		cpu->cycles += 23;
		break;
	}
	case 0xC6: /* SET 0, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x01);
		cpu->cycles += 23;
		break;
	}
	case 0xCE: /* SET 1, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x02);
		cpu->cycles += 23;
		break;
	}
	case 0xD6: /* SET 2, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x04);
		cpu->cycles += 23;
		break;
	}
	case 0xDE: /* SET 3, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x08);
		cpu->cycles += 23;
		break;
	}
	case 0xE6: /* SET 4, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x10);
		cpu->cycles += 23;
		break;
	}
	case 0xEE: /* SET 5, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x20);
		cpu->cycles += 23;
		break;
	}
	case 0xF6: /* SET 6, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x40);
		cpu->cycles += 23;
		break;
	}
	case 0xFE: /* SET 7, (IX+d) */
	{
		int8 d = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->ix.w + d);
		cpu->mwrite(cpu->ix.w + d, m | 0x80);
		cpu->cycles += 23;
		break;
	}
	default:
	{
		printf("Error in emulation, bailing out, opcode = 0xDDCB%02X\n", op2);
		assert(0);
	}
}
