/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(op)	{
	case 0x40: /* IN B, (C) */
	{
		cpu->bc.b.h = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->bc.b.h]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x41: /* OUT (C), B */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->bc.b.h);
		cpu->cycles += 12;
		break;
	}
	case 0x42: /* SBC HL, BC */
	{
		uint32 tmp;
		tmp = cpu->hl.w - (cpu->af.b.l & 0x01) - cpu->bc.w;
		cpu->af.b.l = ((tmp >> 8) & 0x80) | /* S */
			((tmp & 0xFFFF) ? 0x00 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->bc.w) >> 8) & 0x10) |  /* H */
			(((cpu->bc.w ^ cpu->hl.w) & (cpu->hl.w ^ tmp) & 0x8000) >> 13) | /* P */
			0x02 | /* N */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x43: /* LD (nn), BC */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->bc.b.l);
		cpu->mwrite(tmp + 1, cpu->bc.b.h);
		cpu->cycles += 20;
		break;
	}
	case 0x44: /* NEG */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x45: /* RETN */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
	case 0x46: /* IM 0 */
	{
		cpu->im = 0;
		cpu->cycles += 8;
		break;
	}
	case 0x47: /* LD I, A */
	{
		cpu->ir.b.h = cpu->af.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x48: /* IN C, (C) */
	{
		cpu->bc.b.l = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->bc.b.l]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x49: /* OUT (C), C */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->bc.b.l);
		cpu->cycles += 12;
		break;
	}
	case 0x4A: /* ADC HL, BC */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->bc.w + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ((tmp & 0x8000) >> 8) | /* S */
			((tmp & 0xFFFF) ? 0 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->bc.w) & 0x1000) >> 8) | /* H */
			(((cpu->bc.w ^ cpu->hl.w ^ 0x8000) & (cpu->bc.w ^ tmp) & 0x8000) >> 13) | /* P */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x4B: /* LD BC, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->bc.b.l = cpu->mread(tmp++);
		cpu->bc.b.h = cpu->mread(tmp);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x4C: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x4D: /* RETI */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x4E: /* IM 0 - Undocumented */
	{
		cpu->im = 0;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x4F: /* LD R, A */
	{
		cpu->ir.b.l = cpu->af.b.h;
		cpu->r_top = cpu->af.b.h & 0x80;
		cpu->cycles += 9;
		break;
	}
	case 0x50: /* IN D, (C) */
	{
		cpu->de.b.h = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->de.b.h]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x51: /* OUT (C), D */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->de.b.h);
		cpu->cycles += 12;
		break;
	}
	case 0x52: /* SBC HL, DE */
	{
		uint32 tmp;
		tmp = cpu->hl.w - (cpu->af.b.l & 0x01) - cpu->de.w;
		cpu->af.b.l = ((tmp >> 8) & 0x80) | /* S */
			((tmp & 0xFFFF) ? 0x00 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->de.w) >> 8) & 0x10) |  /* H */
			(((cpu->de.w ^ cpu->hl.w) & (cpu->hl.w ^ tmp) & 0x8000) >> 13) | /* P */
			0x02 | /* N */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x53: /* LD (nn), DE */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->de.b.l);
		cpu->mwrite(tmp + 1, cpu->de.b.h);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x54: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x55: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
#endif
	case 0x56: /* IM 1 */
	{
		cpu->im = 1;
		cpu->cycles += 8;
		break;
	}
	case 0x57: /* LD A, I */
	{
		cpu->af.b.h = cpu->ir.b.h;
		cpu->af.b.l = (cpu->ir.b.h & 0x80) | /* S */
			(cpu->ir.b.h ? 0x00 : 0x40) | /* Z */
			(cpu->iff2 << 2) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 9;
		break;
	}
	case 0x58: /* IN E, (C) */
	{
		cpu->de.b.l = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->de.b.l]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x59: /* OUT (C), E */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->de.b.l);
		cpu->cycles += 12;
		break;
	}
	case 0x5A: /* ADC HL, DE */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->de.w + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ((tmp & 0x8000) >> 8) | /* S */
			((tmp & 0xFFFF) ? 0 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->de.w) & 0x1000) >> 8) | /* H */
			(((cpu->de.w ^ cpu->hl.w ^ 0x8000) & (cpu->de.w ^ tmp) & 0x8000) >> 13) | /* P */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x5B: /* LD DE, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->de.b.l = cpu->mread(tmp++);
		cpu->de.b.h = cpu->mread(tmp);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x5C: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x5D: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
#endif
	case 0x5E: /* IM 2 */
	{
		cpu->im = 2;
		cpu->cycles += 8;
		break;
	}
	case 0x5F: /* LD A, R */
	{
		cpu->af.b.h = (cpu->ir.b.l & 0x7F) | cpu->r_top;
		cpu->af.b.l = ZStable[cpu->af.b.h] | /* S, Z */
			(cpu->iff2 << 2) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->cycles += 9;
		break;
	}
	case 0x60: /* IN H, (C) */
	{
		cpu->hl.b.h = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->hl.b.h]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x61: /* OUT (C), H */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->hl.b.h);
		cpu->cycles += 12;
		break;
	}
	case 0x62: /* SBC HL, HL */
	{
		uint32 tmp;
		tmp = cpu->hl.w - (cpu->af.b.l & 0x01) - cpu->hl.w;
		cpu->af.b.l = ((tmp >> 8) & 0x80) | /* S */
			((tmp & 0xFFFF) ? 0x00 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->hl.w) >> 8) & 0x10) |  /* H */
			(((cpu->hl.w ^ cpu->hl.w) & (cpu->hl.w ^ tmp) & 0x8000) >> 13) | /* P */
			0x02 | /* N */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x63: /* LD (nn), HL */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->hl.b.l);
		cpu->mwrite(tmp + 1, cpu->hl.b.h);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x64: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x65: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
	case 0x66: /* IM 0 - Undocumented */
	{
		cpu->im = 0;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x67: /* RRD */
	{
		uint8 m;
		uint8 tmp;
		m = cpu->mread(cpu->hl.w);
		tmp = ((m >> 4) & 0x0F) | ((cpu->af.b.h << 4) & 0xF0);
		cpu->af.b.h = (cpu->af.b.h & 0xF0) | ((m & 0xF0) >> 4);
		cpu->af.b.l = ZSPtable[cpu->af.b.h] | /* Z, S, P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 18;
		break;
	}
	case 0x68: /* IN L, (C) */
	{
		cpu->hl.b.l = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->hl.b.l]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x69: /* OUT (C), L */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->hl.b.l);
		cpu->cycles += 12;
		break;
	}
	case 0x6A: /* ADC HL, HL */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->hl.w + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ((tmp & 0x8000) >> 8) | /* S */
			((tmp & 0xFFFF) ? 0 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->hl.w) & 0x1000) >> 8) | /* H */
			(((cpu->hl.w ^ cpu->hl.w ^ 0x8000) & (cpu->hl.w ^ tmp) & 0x8000) >> 13) | /* P */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x6B: /* LD HL, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->hl.b.l = cpu->mread(tmp++);
		cpu->hl.b.h = cpu->mread(tmp);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x6C: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x6D: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
	case 0x6E: /* IM 0 - Undocumented */
	{
		cpu->im = 0;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x6F: /* RLD */
	{
		uint8 m;
		uint8 tmp;
		m = cpu->mread(cpu->hl.w);
		tmp = ((m & 0x0F) << 4) | (cpu->af.b.h & 0x0F);
		cpu->af.b.h = (cpu->af.b.h & 0xF0) | ((m & 0xF0) >> 4); 
		cpu->af.b.l = ZSPtable[cpu->af.b.h] | /* Z, S, P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 18;
		break;
	}
	case 0x70: /* IN 0, (C) - "Undefined Op Code" */
	{
		/* This one is odd, it reads in from the port,
		   and sets the flags that are appropriate, but
		   doesn't store the byte read in... */
		uint8 tmp = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[tmp]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x71: /* OUT (C), 0 - Undocumented */
	{
		cpu->pwrite(cpu->bc.b.l, 0);
		cpu->cycles += 12;
		break;
	}
#endif
	case 0x72: /* SBC HL, SP */
	{
		uint32 tmp;
		tmp = cpu->hl.w - (cpu->af.b.l & 0x01) - cpu->sp.w;
		cpu->af.b.l = ((tmp >> 8) & 0x80) | /* S */
			((tmp & 0xFFFF) ? 0x00 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->sp.w) >> 8) & 0x10) |  /* H */
			(((cpu->sp.w ^ cpu->hl.w) & (cpu->hl.w ^ tmp) & 0x8000) >> 13) | /* P */
			0x02 | /* N */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x73: /* LD (nn), SP */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->sp.b.l);
		cpu->mwrite(tmp + 1, cpu->sp.b.h);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x74: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x75: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
	case 0x76: /* IM 1 - Undocumented */
	{
		cpu->im = 1;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0x78: /* IN A, (C) */
	{
		cpu->af.b.h = cpu->pread(cpu->bc.b.l);
		cpu->af.b.l = (cpu->af.b.l & 0x01) | /* C */
			ZSPtable[cpu->af.b.h]; /* Z, S, P */
		cpu->cycles += 12;
		break;
	}
	case 0x79: /* OUT (C), A */
	{
		cpu->pwrite(cpu->bc.b.l, cpu->af.b.h);
		cpu->cycles += 12;
		break;
	}
	case 0x7A: /* ADC HL, SP */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->sp.w + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ((tmp & 0x8000) >> 8) | /* S */
			((tmp & 0xFFFF) ? 0 : 0x40) | /* Z */
			(((cpu->hl.w ^ tmp ^ cpu->sp.w) & 0x1000) >> 8) | /* H */
			(((cpu->sp.w ^ cpu->hl.w ^ 0x8000) & (cpu->sp.w ^ tmp) & 0x8000) >> 13) | /* P */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x7B: /* LD SP, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->sp.b.l = cpu->mread(tmp++);
		cpu->sp.b.h = cpu->mread(tmp);
		cpu->cycles += 20;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x7C: /* NEG - Undocumented */
	{
		uint32 tmp;
		tmp = 0 - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((0 ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ 0) & (0 ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 8;
		break;
	}
	case 0x7D: /* RETN - Undocumented */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->iff1 = cpu->iff2;
		cpu->cycles += 14;
		break;
	}
	case 0x7E: /* IM 1 - Undocumented */
	{
		cpu->im = 1;
		cpu->cycles += 8;
		break;
	}
#endif
	case 0xA0: /* LDI */
	{
		cpu->mwrite(cpu->de.w++, cpu->mread(cpu->hl.w++));
		--cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC1) | /* S, Z, C */
			(cpu->bc.w ? 0x04 : 0x00); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xA1: /* CPI */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w++);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			(cpu->af.b.l & 0x01) | /* C */
			(cpu->bc.w ? 0x04 : 0x00) | /* P */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* H */
			0x02; /* N */
		--cpu->bc.w;
		cpu->cycles += 16;
		break;
	}
	case 0xA2: /* INI */
	{
		uint8 m = cpu->pread(cpu->bc.b.l);
		cpu->mwrite(cpu->hl.w++, m);
		--cpu->bc.b.h;
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((m + ((cpu->bc.b.l + 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((m + ((cpu->bc.b.l + 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xA3: /* OUTI */
	{
		uint8 m = cpu->mread(cpu->hl.w++);
		--cpu->bc.b.h;
		cpu->pwrite(cpu->bc.b.l, m);
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((cpu->hl.b.l + m) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((cpu->hl.b.l + m) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xA8: /* LDD */
	{
		uint8 m = cpu->mread(cpu->hl.w--);
		cpu->mwrite(cpu->de.w--, m);
		--cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC1) | /* S, Z, C */
			(cpu->bc.w ? 0x04 : 0x00); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xA9: /* CPD */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w--);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			(cpu->af.b.l & 0x01) | /* C */
			(cpu->bc.w ? 0x04 : 0x00) | /* P */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* H */
			0x02; /* N */
		--cpu->bc.w;
		cpu->cycles += 16;
		break;
	}
	case 0xAA: /* IND */
	{
		uint8 m = cpu->pread(cpu->bc.b.l);
		cpu->mwrite(cpu->hl.w--, m);
		--cpu->bc.b.h;
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((m + ((cpu->bc.b.l - 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((m + ((cpu->bc.b.l - 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xAB: /* OUTD */
	{
		uint8 m = cpu->mread(cpu->hl.w--);
		--cpu->bc.b.h;
		cpu->pwrite(cpu->bc.b.l, m);
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((cpu->hl.b.l + m) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((cpu->hl.b.l + m) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		cpu->cycles += 16;
		break;
	}
	case 0xB0: /* LDIR */
	{
		uint8 m = cpu->mread(cpu->hl.w++);
		cpu->mwrite(cpu->de.w++, m);
		--cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC1) | /* S, Z, C */
			(cpu->bc.w ? 0x04 : 0x00); /* P */
		if(!cpu->bc.w)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	case 0xB1: /* CPIR */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w++);
		tmp = cpu->af.b.h - m;
		--cpu->bc.w;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			(cpu->af.b.l & 0x01) | /* C */
			(cpu->bc.w ? 0x04 : 0x00) | /* P */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* H */
			0x02; /* N */
		if(cpu->bc.w && tmp)	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		else	{
			cpu->cycles += 16;
		}
		break;
	}
	case 0xB2: /* INIR */
	{
		uint8 m = cpu->pread(cpu->bc.b.l);
		cpu->mwrite(cpu->hl.w++, m);
		--cpu->bc.b.h;
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((m + ((cpu->bc.b.l + 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((m + ((cpu->bc.b.l + 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		if(!cpu->bc.b.h)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	case 0xB3: /* OTIR */
	{
		uint8 m = cpu->mread(cpu->hl.w++);
		--cpu->bc.b.h;
		cpu->pwrite(cpu->bc.b.l, m);
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((cpu->hl.b.l + m) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((cpu->hl.b.l + m) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		if(!cpu->bc.b.h)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	case 0xB8: /* LDDR */
	{
		uint8 m = cpu->mread(cpu->hl.w--);
		cpu->mwrite(cpu->de.w--, m);
		--cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC1) | /* S, Z, C */
			(cpu->bc.w ? 0x04 : 0x00); /* P */
		if(!cpu->bc.w)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	case 0xB9: /* CPDR */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w--);
		tmp = cpu->af.b.h - m;
		--cpu->bc.w;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			(cpu->af.b.l & 0x01) | /* C */
			(cpu->bc.w ? 0x04 : 0x00) | /* P */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* H */
			0x02; /* N */
		if(cpu->bc.w && tmp)	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		else	{
			cpu->cycles += 16;
		}
		break;
	}
	case 0xBA: /* INDR */
	{
		uint8 m = cpu->pread(cpu->bc.b.l);
		cpu->mwrite(cpu->hl.w--, m);
		--cpu->bc.b.h;
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((m + ((cpu->bc.b.l - 1) & 0xFF)) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((m + ((cpu->bc.b.l - 1) & 0xFF)) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		if(!cpu->bc.b.h)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	case 0xBB: /* OTDR */
	{
		uint8 m = cpu->mread(cpu->hl.w--);
		--cpu->bc.b.h;
		cpu->pwrite(cpu->bc.b.l, m);
		cpu->af.b.l = ZStable[cpu->bc.b.h] | /* S, Z */
			((m >> 6) & 0x02) | /* N */
			((cpu->hl.b.l + m) > 0xFF ? 0x11 : 0x00) | /* H, C */
			(ZSPtable[((cpu->hl.b.l + m) & 0x07) ^ cpu->bc.b.h] & 0x04); /* P */
		if(!cpu->bc.b.h)	{
			cpu->cycles += 16;
		}
		else	{
			cpu->pc.w -= 2;
			cpu->cycles += 21;
		}
		break;
	}
	default:
	{
		printf("Illegal opcode, bailing out, opcode = 0xED%02X\n", op);
		assert(0);
		break;
	}
}
