/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(op)	{
	case 0x09: /* ADD IY, BC */
	{
		uint32 tmp;
		tmp = cpu->iy.w + cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->iy.w ^ tmp ^ cpu->bc.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->iy.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x19: /* ADD IY, DE */
	{
		uint32 tmp;
		tmp = cpu->iy.w + cpu->de.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->iy.w ^ tmp ^ cpu->de.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->iy.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x21: /* LD IY, nn */
	{
		cpu->iy.b.l = cpu->mread(cpu->pc.w++);
		cpu->iy.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 14;
		break;
	}
	case 0x22: /* LD (nn), IY */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->iy.b.l);
		cpu->mwrite(tmp + 1, cpu->iy.b.h);
		cpu->cycles += 20;
		break;
	}
	case 0x23: /* INC IY */
	{
		++cpu->iy.w;
		cpu->cycles += 10;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x24: /* INC IYh */
	{
		uint32 tmp = cpu->iy.b.h + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->iy.b.h == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->iy.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x25: /* DEC IYh */
	{
		uint32 tmp = cpu->iy.b.h - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->iy.b.h ^ tmp) & 0x10) | /* H */
			(cpu->iy.b.h == 0x80 ? 0x04 : 0x00) | /* P */
			0x02 | /* N */
			(cpu->af.b.l & 0x01); /* C */
		cpu->iy.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x26: /* LD IYh, n */
	{
		cpu->iy.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x29: /* ADD IY, IY */
	{
		uint32 tmp;
		tmp = cpu->iy.w + cpu->iy.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->iy.w ^ tmp ^ cpu->iy.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->iy.w = tmp;
		cpu->cycles += 15;
		break;
	}
	case 0x2A: /* LD IY, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->iy.b.l = cpu->mread(tmp);
		cpu->iy.b.h = cpu->mread(tmp + 1);
		cpu->cycles += 20;
		break;
	}
	case 0x2B: /* DEC IY */
	{
		--cpu->iy.w;
		cpu->cycles += 10;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x2C: /* INC IYl */
	{
		uint32 tmp = cpu->iy.b.l + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->iy.b.l == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->iy.b.l = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x2D: /* DEC IYl */
	{
		uint32 tmp = cpu->iy.b.l - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->iy.b.l ^ tmp) & 0x10) | /* H */
			(cpu->iy.b.l == 0x80 ? 0x04 : 0x00) | /* P */
			0x02 | /* N */
			(cpu->af.b.l & 0x01); /* C */
		cpu->iy.b.l = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x2E: /* LD IYl, n */
	{
		cpu->iy.b.l = cpu->mread(cpu->pc.w++);
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x34: /* INC (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp);
		uint32 tmp2 = m + 1;
		cpu->af.b.l = ZStable[tmp2 & 0xFF] | /* S, Z */
			((tmp2 & 0x0F) ? 0x00 : 0x10) | /* H */
			((m == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->iy.w + tmp, tmp2 & 0xFF);
		cpu->cycles += 23;
		break;
	}
	case 0x35: /* DEC (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp);
		uint32 tmp2 = m - 1;
		cpu->af.b.l = ZStable[tmp2 & 0xFF] | /* S, Z */
			((m ^ tmp2) & 0x10) | /* H */
			(m == 0x80 ? 0x04 : 0x00) | /* P */
			0x02 | /* N */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->iy.w + tmp, tmp2 & 0xFF);
		cpu->cycles += 23;
		break;
	}
	case 0x36: /* LD (IY+d), n */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->pc.w++);
		cpu->mwrite(tmp + cpu->iy.w, m);
		cpu->cycles += 19;
		break;
	}
	case 0x39: /* ADD IY, SP */
	{
		uint32 tmp;
		tmp = cpu->iy.w + cpu->sp.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->iy.w ^ tmp ^ cpu->sp.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->iy.w = tmp;
		cpu->cycles += 15;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x44: /* LD B, IYh */
	{
		cpu->bc.b.h = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x45: /* LD B, IYl */
	{
		cpu->bc.b.h = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x46: /* LD B, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->bc.b.h = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x4C: /* LD C, IYh */
	{
		cpu->bc.b.l = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x4D: /* LD C, IYl */
	{
		cpu->bc.b.l = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x4E: /* LD C, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->bc.b.l = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x54: /* LD D, IYh */
	{
		cpu->de.b.h = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x55: /* LD D, IYl */
	{
		cpu->de.b.h = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x56: /* LD D, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->de.b.h = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x5C: /* LD E, IYh */
	{
		cpu->de.b.l = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x5D: /* LD E, IYl */
	{
		cpu->de.b.l = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x5E: /* LD E, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->de.b.l = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x60: /* LD IYh, B */
	{
		cpu->iy.b.h = cpu->bc.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x61: /* LD IYh, C */
	{
		cpu->iy.b.h = cpu->bc.b.l;
		cpu->cycles += 9;
		break;
	}
	case 0x62: /* LD IYh, D */
	{
		cpu->iy.b.h = cpu->de.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x63: /* LD IYh, E */
	{
		cpu->iy.b.h = cpu->de.b.l;
		cpu->cycles += 9;
		break;
	}
	case 0x64: /* LD IYh, IYh */
	{
		cpu->cycles += 9;
		break;
	}
	case 0x65: /* LD IYh, IYl */
	{
		cpu->iy.b.h = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x66: /* LD H, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->hl.b.h = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x67: /* LD IYh, A */
	{
		cpu->iy.b.h = cpu->af.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x68: /* LD IYl, B */
	{
		cpu->iy.b.l = cpu->bc.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x69: /* LD IYl, C */
	{
		cpu->iy.b.l = cpu->bc.b.l;
		cpu->cycles += 9;
		break;
	}
	case 0x6A: /* LD IYl, D */
	{
		cpu->iy.b.l = cpu->de.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x6B: /* LD IYl, E */
	{
		cpu->iy.b.l = cpu->de.b.l;
		cpu->cycles += 9;
		break;
	}
	case 0x6C: /* LD IYl, IYh */
	{
		cpu->iy.b.l = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x6D: /* LD IYl, IYl */
	{
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x6E: /* LD L, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->hl.b.l = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x6F: /* LD IYl, A */
	{
		cpu->iy.b.l = cpu->af.b.h;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x70: /* LD (IY+d), B */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->bc.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x71: /* LD (IY+d), C */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->bc.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x72: /* LD (IY+d), D */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->de.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x73: /* LD (IY+d), E */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->de.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x74: /* LD (IY+d), H */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->hl.b.h);
		cpu->cycles += 19;
		break;
	}
	case 0x75: /* LD (IY+d), L */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->hl.b.l);
		cpu->cycles += 19;
		break;
	}
	case 0x77: /* LD (IY+d), A */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->mwrite(cpu->iy.w + tmp, cpu->af.b.h);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x7C: /* LD A, IYh */
	{
		cpu->af.b.h = cpu->iy.b.h;
		cpu->cycles += 9;
		break;
	}
	case 0x7D: /* LD A, IYl */
	{
		cpu->af.b.h = cpu->iy.b.l;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x7E: /* LD A, (IY+d) */
	{
		int8 tmp = (int8) cpu->mread(cpu->pc.w++);
		cpu->af.b.h = cpu->mread(cpu->iy.w + tmp);
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x84: /* ADD A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->iy.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.h) & 0x10) | /* H */
			(((cpu->iy.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->iy.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x85: /* ADD A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->iy.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.l) & 0x10) | /* H */
			(((cpu->iy.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->iy.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x86: /* ADD A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h + m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x8C: /* ADC A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->iy.b.h + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.h) & 0x10) | /* H */
			(((cpu->iy.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->iy.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x8D: /* ADC A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->iy.b.l + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.l) & 0x10) | /* H */
			(((cpu->iy.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->iy.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x8E: /* ADC A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h + m + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x94: /* SUB A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->iy.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.h) & 0x10) | /* H */
			(((cpu->iy.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x95: /* SUB A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->iy.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.l) & 0x10) | /* H */
			(((cpu->iy.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x96: /* SUB A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			((tmp >> 8) & 0x01) | /* Set the carry flag */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* Overflow */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0x9C: /* SBC A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->iy.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.h) & 0x10) | /* H */
			(((cpu->iy.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0x9D: /* SBC A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->iy.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.l) & 0x10) | /* H */
			(((cpu->iy.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0x9E: /* SBC A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (m ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp & 0xFF;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xA4: /* AND A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->iy.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0xA5: /* AND A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->iy.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0xA6: /* AND A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h & m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			0x10; /* Set the Half-carry flag */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xAC: /* XOR A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->iy.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0xAD: /* XOR A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->iy.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0xAE: /* XOR A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h ^ m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xB4: /* OR A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->iy.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
	case 0xB5: /* OR A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->iy.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 9;
		break;
	}
#endif
	case 0xB6: /* OR A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = cpu->mread(cpu->pc.w++);
        uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h | m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 19;
		break;
	}
#ifndef CRABZ80_NO_UNDOCUMENTED_OPS
	case 0xBC: /* CP A, IYh */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->iy.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.h) & 0x10) | /* H */
			(((cpu->iy.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBD: /* CP A, IYl */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->iy.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->iy.b.l) & 0x10) | /* H */
			(((cpu->iy.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
#endif
	case 0xBE: /* CP A, (IY+d) */
	{
		uint32 tmp;
		int8 tmp2 = (int8) cpu->mread(cpu->pc.w++);
		uint8 m = cpu->mread(cpu->iy.w + tmp2);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 19;
		break;
	}
	case 0xCB: /* FDCB prefiyed opcodes */
	{
		uint8 op2 = cpu->mread(cpu->pc.w + 1);

#		include "z80_switch_FDCB.h"

		++cpu->pc.w;

		break;
	}
	case 0xE1: /* POP IY */
	{
		cpu->iy.b.l = cpu->mread(cpu->sp.w++);
		cpu->iy.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 14;
		break;
	}
	case 0xE3: /* EX (SP), IY */
	{
		uint8 tmp, tmp2;
		tmp = cpu->iy.b.l;
		tmp2 = cpu->iy.b.h;
		cpu->iy.b.l = cpu->mread(cpu->sp.w++);
		cpu->iy.b.h = cpu->mread(cpu->sp.w);
		cpu->mwrite(cpu->sp.w, tmp2);
		cpu->mwrite(--cpu->sp.w, tmp);
		cpu->cycles += 23;
		break;
	}
	case 0xE5: /* PUSH IY */
	{
		cpu->mwrite(--cpu->sp.w, cpu->iy.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->iy.b.l);
		cpu->cycles += 15;
		break;
	}
	case 0xE9: /* JP (IY) (Really JP IY) */
	{
		cpu->pc.w = cpu->iy.w;
		cpu->cycles += 8;
		break;
	}
	case 0xF9: /* LD SP, IY */
	{
		cpu->sp.w = cpu->iy.w;
		cpu->cycles += 10;
		break;
	}
	default:
	{
		printf("Error in emulation, bailing out, opcode = 0xFD%02X\n", op);
		assert(0);
	}
}

