/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

switch(inst)	{
	case 0x00: /* NOP */
	{
		cpu->cycles += 4;
		break;
	}
	case 0x01: /* LD BC, nn */
	{
		uint16 tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->bc.w = tmp;
		cpu->cycles += 10;
		break;
	}
	case 0x02: /* LD (BC), A */
	{
		cpu->mwrite(cpu->bc.w, cpu->af.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x03: /* INC BC */
	{
		++cpu->bc.w;
		cpu->cycles += 6;
		break;
	}
	case 0x04: /* INC B */
	{
		uint32 tmp = cpu->bc.b.h + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->bc.b.h == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->bc.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x05: /* DEC B */
	{
		uint32 tmp = cpu->bc.b.h - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->bc.b.h == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->bc.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->bc.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x06: /* LD B, n */
	{
		cpu->bc.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x07: /* RLCA */
	{
		uint8 tmp = ((cpu->af.b.h & 0x80) >> 7) & 0x01;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(tmp); /* C */
		cpu->af.b.h = ((cpu->af.b.h << 1) & 0xFE) | tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x08: /* EX AF, AF' */
	{
		uint16 tmp = cpu->af.w;
		cpu->af.w = cpu->afp.w;
		cpu->afp.w = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x09: /* ADD HL, BC */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->bc.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->hl.w ^ tmp ^ cpu->bc.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 11;
		break;
	}
	case 0x0A: /* LD A, (BC) */
	{
		cpu->af.b.h = cpu->mread(cpu->bc.w);
		cpu->cycles += 7;
		break;
	}
	case 0x0B: /* DEC BC */
	{
		--cpu->bc.w;
		cpu->cycles += 4;
		break;
	}
	case 0x0C: /* INC C */
	{
		uint32 tmp = cpu->bc.b.l + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->bc.b.l == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->bc.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x0D: /* DEC C */
	{
		uint32 tmp = cpu->bc.b.l - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->bc.b.l == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->bc.b.l ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->bc.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x0E: /* LD C, n */
	{
		cpu->bc.b.l = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x0F: /* RRCA */
	{
		uint8 bot = cpu->af.b.h & 0x01;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(bot); /* C */
		cpu->af.b.h = (cpu->af.b.h >> 1) | (bot << 7);
		cpu->cycles += 4;
		break;
	}
	case 0x10: /* DJNZ e */
	{
		if(--cpu->bc.b.h)	{
			/* B != 0, jump */
			cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
			cpu->cycles += 13;
		}
		else	{
			++cpu->pc.w; /* Increment past the offset */
			cpu->cycles += 8;
		}
		break;
	}
	case 0x11: /* LD DE, nn */
	{
		uint16 tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->de.w = tmp;
		cpu->cycles += 10;
		break;
	}
	case 0x12: /* LD (DE), A */
	{
		cpu->mwrite(cpu->de.w, cpu->af.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x13: /* INC DE */
	{
		++cpu->de.w;
		cpu->cycles += 6;
		break;
	}
	case 0x14: /* INC D */
	{
		uint32 tmp = cpu->de.b.h + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->de.b.h == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->de.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x15: /* DEC D */
	{
		uint32 tmp = cpu->de.b.h - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->de.b.h == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->de.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->de.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x16: /* LD D, n */
	{
		cpu->de.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x17: /* RLA */
	{
		uint32 tmp = (cpu->af.b.h << 1) | (cpu->af.b.l & 0x01);
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			((tmp >> 8) & 0x01);
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x18: /* JR e */
	{
		cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
		cpu->cycles += 12;
		break;
	}
	case 0x19: /* ADD HL, DE */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->de.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->hl.w ^ tmp ^ cpu->de.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 11;
		break;
	}
	case 0x1A: /* LD A, (DE) */
	{
		cpu->af.b.h = cpu->mread(cpu->de.w);
		cpu->cycles += 7;
		break;
	}
	case 0x1B: /* DEC DE */
	{
		--cpu->de.w;
		cpu->cycles += 6;
		break;
	}
	case 0x1C: /* INC E */
	{
		uint32 tmp = cpu->de.b.l + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->de.b.l == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->de.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x1D: /* DEC E */
	{
		uint32 tmp = cpu->de.b.l - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->de.b.l == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->de.b.l ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->de.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x1E: /* LD E, n */
	{
		cpu->de.b.l = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x1F: /* RRA */
	{
		uint32 tmp = (cpu->af.b.h >> 1) | ((cpu->af.b.l & 0x01) << 7);
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(cpu->af.b.h & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x20: /* JR NZ, e */
	{
		if(cpu->af.b.l & 0x40)	{ 
			/* Z = 1, no jump */
			++cpu->pc.w; /* Increment past the offset */
			cpu->cycles += 7;
		}
		else	{
			//tmp.w = cpu->pc.w + 1 + cpu->mread(cpu->pc.w);
			//cpu->pc.w = tmp.w;
			cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
			cpu->cycles += 12;
		}
		break;
	}
	case 0x21: /* LD HL, nn */
	{
		uint16 tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->hl.w = tmp;
		cpu->cycles += 10;
		break;
	}
	case 0x22: /* LD (nn), HL */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->hl.b.l);
		cpu->mwrite(tmp + 1, cpu->hl.b.h);
		cpu->cycles += 16;
		break;
	}
	case 0x23: /* INC HL */
	{
		++cpu->hl.w;
		cpu->cycles += 6;
		break;
	}
	case 0x24: /* INC H */
	{
		uint32 tmp = cpu->hl.b.h + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->hl.b.h == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->hl.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x25: /* DEC H */
	{
		uint32 tmp = cpu->hl.b.h - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->hl.b.h == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->hl.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->hl.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x26: /* LD H, n */
	{
		cpu->hl.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x27: /* DAA */
	{
		int index = cpu->af.b.h;
		index |= ((cpu->af.b.l & 0x01) ? 0x100 : 0x000) | /* C */
			((cpu->af.b.l & 0x10) ? 0x200 : 0x000) | /* H */
			((cpu->af.b.l & 0x02) ? 0x400 : 0x000); /* N */
		cpu->af.w = DAAtable[index];
		cpu->cycles += 4;
		break;
	}
	case 0x28: /* JR Z, e */
	{
		if(cpu->af.b.l & 0x40)	{ 
			/* Z = 1, jump */
			cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
			cpu->cycles += 12;
		}
		else	{
			++cpu->pc.w; /* Increment past the offset */
			cpu->cycles += 7;
		}
		break;
	}
	case 0x29: /* ADD HL, HL */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->hl.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->hl.w ^ tmp ^ cpu->hl.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 11;
		break;
	}
	case 0x2A: /* LD HL, (nn) */
	{
		uint16 tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->hl.b.l = cpu->mread(tmp);
		cpu->hl.b.h = cpu->mread(tmp + 1);
		cpu->cycles += 16;
		break;
	}
	case 0x2B: /* DEC HL */
	{
		--cpu->hl.w;
		cpu->cycles += 6;
		break;
	}
	case 0x2C: /* INC L */
	{
		uint32 tmp = cpu->hl.b.l + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->hl.b.l == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->hl.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x2D: /* DEC L */
	{
		uint32 tmp = cpu->hl.b.l - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->hl.b.l == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->hl.b.l ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->hl.b.l = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x2E: /* LD L, n */
	{
		cpu->hl.b.l = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x2F: /* CPL */
	{
		cpu->af.b.h = ~cpu->af.b.h;
		cpu->af.b.l |= 0x12;
		cpu->cycles += 4;
		break;
	}
	case 0x30: /* JR NC, e */
	{
		if(cpu->af.b.l & 0x01)	{ 
			/* C = 1, no jump */
			++cpu->pc.w; /* Increment past the offset */
			cpu->cycles += 7;
		}
		else	{
			cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
			cpu->cycles += 12;
		}
		break;
	}
	case 0x31: /* LD SP, nn */
	{
		uint16 tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->sp.w = tmp;
		cpu->cycles += 10;
		break;
	}
	case 0x32: /* LD nn, A */
	{
		uint16 tmp = (cpu->mread(cpu->pc.w++)) | (cpu->mread(cpu->pc.w++) << 8);
		cpu->mwrite(tmp, cpu->af.b.h);
		cpu->cycles += 13;
		break;
	}
	case 0x33: /* INC SP */
	{
		++cpu->sp.w;
		cpu->cycles += 6;
		break;
	}
	case 0x34: /* INC (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint32 tmp = m + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((m == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->mwrite(cpu->hl.w, tmp & 0xFF);
		cpu->cycles += 11;
		break;
	}
	case 0x35: /* DEC (HL) */
	{
		uint8 m = cpu->mread(cpu->hl.w);
		uint32 tmp = m - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(m == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((m ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->mwrite(cpu->hl.w, tmp);
		cpu->cycles += 10;
		break;
	}
	case 0x36: /* LD (HL), n */
	{
		cpu->mwrite(cpu->hl.w, cpu->mread(cpu->pc.w++));
		cpu->cycles += 10;
		break;
	}
	case 0x37: /* SCF */
	{
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(0x01); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0x38: /* JR C, e */
	{
		if(cpu->af.b.l & 0x01)	{ 
			/* C = 1, jump */
			cpu->pc.w += (int8) cpu->mread(cpu->pc.w) + 1;
			cpu->cycles += 12;
		}
		else	{
			++cpu->pc.w; /* Increment past the offset */
			cpu->cycles += 7;
		}
		break;
	}
	case 0x39: /* ADD HL, SP */
	{
		uint32 tmp;
		tmp = cpu->hl.w + cpu->sp.w;
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			(((cpu->hl.w ^ tmp ^ cpu->sp.w) & 0x1000) >> 8) | /* H */
			((tmp >> 16) & 0x01); /* C */
		cpu->hl.w = tmp;
		cpu->cycles += 11;
		break;
	}
	case 0x3A: /* LD A, (nn) */
	{
		uint16 tmp;
		tmp = cpu->mread(cpu->pc.w++) | (cpu->mread(cpu->pc.w++) << 8);
		//cpu->mwrite(tmp, cpu->af.b.h);
		cpu->af.b.h = cpu->mread(tmp);
		cpu->cycles += 13;
		break;
	}
	case 0x3B: /* DEC SP */
	{
		--cpu->sp.w;
		cpu->cycles += 6;
		break;
	}
	case 0x3C: /* INC A */
	{
		uint32 tmp = cpu->af.b.h + 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((tmp & 0x0F) ? 0x00 : 0x10) | /* H */
			((cpu->af.b.h == 0x7F) ? 0x04: 0x00) | /* P */
			(cpu->af.b.l & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x3D: /* DEC A */
	{
		uint32 tmp = cpu->af.b.h - 1;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			(cpu->af.b.h == 0x80 ? 0x04 : 0x00) | /* Overflow/Parity */
			(cpu->af.b.l & 0x01) | /* Carry flag */
			((cpu->af.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x3E: /* LD A, n */
	{
		cpu->af.b.h = cpu->mread(cpu->pc.w++);
		cpu->cycles += 7;
		break;
	}
	case 0x3F: /* CCF */
	{
		cpu->af.b.l = (cpu->af.b.l & 0xC4) | /* S, Z, P */
			((cpu->af.b.l & 0x01) ? 0x10 : 0x01); /* H, C */
		cpu->cycles += 4;
		break;
	}
	case 0x40: /* LD B, B */
	{
		cpu->bc.b.h = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x41: /* LD B, C */
	{
		cpu->bc.b.h = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x42: /* LD B, D */
	{
		cpu->bc.b.h = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x43: /* LD B, E */
	{
		cpu->bc.b.h = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x44: /* LD B, H */
	{
		cpu->bc.b.h = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x45: /* LD B, L */
	{
		cpu->bc.b.h = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x46: /* LD B, (HL) */
	{
		cpu->bc.b.h = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x47: /* LD B, A */
	{
		cpu->bc.b.h = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x48: /* LD C, B */
	{
		cpu->bc.b.l = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x49: /* LD C, C */
	{
		cpu->bc.b.l = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x4A: /* LD C, D */
	{
		cpu->bc.b.l = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x4B: /* LD C, E */
	{
		cpu->bc.b.l = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x4C: /* LD C, H */
	{
		cpu->bc.b.l = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x4D: /* LD C, L */
	{
		cpu->bc.b.l = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x4E: /* LD C, (HL) */
	{
		cpu->bc.b.l = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x4F: /* LD C, A */
	{
		cpu->bc.b.l = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x50: /* LD D, B */
	{
		cpu->de.b.h = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x51: /* LD D, C */
	{
		cpu->de.b.h = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x52: /* LD D, D */
	{
		cpu->de.b.h = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x53: /* LD D, E */
	{
		cpu->de.b.h = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x54: /* LD D, H */
	{
		cpu->de.b.h = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x55: /* LD D, L */
	{
		cpu->de.b.h = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x56: /* LD D, (HL) */
	{
		cpu->de.b.h = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x57: /* LD D, A */
	{
		cpu->de.b.h = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x58: /* LD E, B */
	{
		cpu->de.b.l = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x59: /* LD E, C */
	{
		cpu->de.b.l = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x5A: /* LD E, D */
	{
		cpu->de.b.l = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x5B: /* LD E, E */
	{
		cpu->de.b.l = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x5C: /* LD E, H */
	{
		cpu->de.b.l = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x5D: /* LD E, L */
	{
		cpu->de.b.l = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x5E: /* LD E, (HL) */
	{
		cpu->de.b.l = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x5F: /* LD E, A */
	{
		cpu->de.b.l = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x60: /* LD H, B */
	{
		cpu->hl.b.h = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x61: /* LD H, C */
	{
		cpu->hl.b.h = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x62: /* LD H, D */
	{
		cpu->hl.b.h = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x63: /* LD H, E */
	{
		cpu->hl.b.h = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x64: /* LD H, H */
	{
		cpu->hl.b.h = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x65: /* LD H, L */
	{
		cpu->hl.b.h = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x66: /* LD H, (HL) */
	{
		cpu->hl.b.h = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x67: /* LD H, A */
	{
		cpu->hl.b.h = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x68: /* LD L, B */
	{
		cpu->hl.b.l = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x69: /* LD L, C */
	{
		cpu->hl.b.l = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x6A: /* LD L, D */
	{
		cpu->hl.b.l = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x6B: /* LD L, E */
	{
		cpu->hl.b.l = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x6C: /* LD L, H */
	{
		cpu->hl.b.l = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x6D: /* LD L, L */
	{
		cpu->hl.b.l = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x6E: /* LD L, (HL) */
	{
		cpu->hl.b.l = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x6F: /* LD L, A */
	{
		cpu->hl.b.l = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x70: /* LD (HL), B */
	{
		cpu->mwrite(cpu->hl.w, cpu->bc.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x71: /* LD (HL), C */
	{
		cpu->mwrite(cpu->hl.w, cpu->bc.b.l);
		cpu->cycles += 7;
		break;
	}
	case 0x72: /* LD (HL), D */
	{
		cpu->mwrite(cpu->hl.w, cpu->de.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x73: /* LD (HL), E */
	{
		cpu->mwrite(cpu->hl.w, cpu->de.b.l);
		cpu->cycles += 7;
		break;
	}
	case 0x74: /* LD (HL), H */
	{
		cpu->mwrite(cpu->hl.w, cpu->hl.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x75: /* LD (HL), L */
	{
		cpu->mwrite(cpu->hl.w, cpu->hl.b.l);
		cpu->cycles += 7;
		break;
	}
	case 0x76: /* HALT */
	{
		--cpu->pc.w;
		cpu->halt = 1;
		cpu->cycles += 4;
		break;
	}
	case 0x77: /* LD (HL), A */
	{
		cpu->mwrite(cpu->hl.w, cpu->af.b.h);
		cpu->cycles += 7;
		break;
	}
	case 0x78: /* LD A, B */
	{
		cpu->af.b.h = cpu->bc.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x79: /* LD A, C */
	{
		cpu->af.b.h = cpu->bc.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x7A: /* LD A, D */
	{
		cpu->af.b.h = cpu->de.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x7B: /* LD A, E */
	{
		cpu->af.b.h = cpu->de.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x7C: /* LD A, H */
	{
		cpu->af.b.h = cpu->hl.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x7D: /* LD A, L */
	{
		cpu->af.b.h = cpu->hl.b.l;
		cpu->cycles += 4;
		break;
	}
	case 0x7E: /* LD A, (HL) */
	{
		cpu->af.b.h = cpu->mread(cpu->hl.w);
		cpu->cycles += 7;
		break;
	}
	case 0x7F: /* LD A, A */
	{
		cpu->af.b.h = cpu->af.b.h;
		cpu->cycles += 4;
		break;
	}
	case 0x80: /* ADD A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->bc.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.h) & 0x10) | /* H */
			(((cpu->bc.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->bc.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x81: /* ADD A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->bc.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.l) & 0x10) | /* H */
			(((cpu->bc.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->bc.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x82: /* ADD A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->de.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.h) & 0x10) | /* H */
			(((cpu->de.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->de.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x83: /* ADD A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->de.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.l) & 0x10) | /* H */
			(((cpu->de.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->de.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x84: /* ADD A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->hl.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.h) & 0x10) | /* H */
			(((cpu->hl.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->hl.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x85: /* ADD A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->hl.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.l) & 0x10) | /* H */
			(((cpu->hl.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->hl.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x86: /* ADD A, (HL) */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h + m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0x87: /* ADD A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x88: /* ADC A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->bc.b.h + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.h) & 0x10) | /* H */
			(((cpu->bc.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->bc.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x89: /* ADC A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->bc.b.l + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.l) & 0x10) | /* H */
			(((cpu->bc.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->bc.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x8A: /* ADC A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->de.b.h + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.h) & 0x10) | /* H */
			(((cpu->de.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->de.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x8B: /* ADC A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->de.b.l + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.l) & 0x10) | /* H */
			(((cpu->de.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->de.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x8C: /* ADC A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->hl.b.h + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.h) & 0x10) | /* H */
			(((cpu->hl.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->hl.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x8D: /* ADC A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->hl.b.l + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.l) & 0x10) | /* H */
			(((cpu->hl.b.l ^ cpu->af.b.h ^ 0x80) & (cpu->hl.b.l ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x8E: /* ADC A, (HL) */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h + m + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0x8F: /* ADC A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h + cpu->af.b.h + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ cpu->af.b.h ^ 0x80) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x90: /* SUB A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->bc.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.h) & 0x10) | /* H */
			(((cpu->bc.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x91: /* SUB A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->bc.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.l) & 0x10) | /* H */
			(((cpu->bc.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x92: /* SUB A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->de.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.h) & 0x10) | /* H */
			(((cpu->de.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x93: /* SUB A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->de.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.l) & 0x10) | /* H */
			(((cpu->de.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x94: /* SUB A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->hl.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.h) & 0x10) | /* H */
			(((cpu->hl.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x95: /* SUB A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->hl.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.l) & 0x10) | /* H */
			(((cpu->hl.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x96: /* SUB A, (HL) */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0x97: /* SUB A, A */
	{
		uint32 tmp = cpu->af.b.h - cpu->af.b.h;
		cpu->af.b.h = 0; /* any number minus itself = 0 */
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0x98: /* SBC A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->bc.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.h) & 0x10) | /* H */
			(((cpu->bc.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x99: /* SBC A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->bc.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.l) & 0x10) | /* H */
			(((cpu->bc.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x9A: /* SBC A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->de.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.h) & 0x10) | /* H */
			(((cpu->de.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x9B: /* SBC A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->de.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.l) & 0x10) | /* H */
			(((cpu->de.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x9C: /* SBC A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->hl.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.h) & 0x10) | /* H */
			(((cpu->hl.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x9D: /* SBC A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->hl.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.l) & 0x10) | /* H */
			(((cpu->hl.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0x9E: /* SBC A, (HL) */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0x9F: /* SBC A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0xA0: /* AND A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->bc.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA1: /* AND A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->bc.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA2: /* AND A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->de.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA3: /* AND A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->de.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA4: /* AND A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->hl.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA5: /* AND A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->hl.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA6: /* AND A, (HL) */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h & m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
    case 0xA7: /* AND A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h & cpu->af.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA8: /* XOR A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->bc.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xA9: /* XOR A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->bc.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xAA: /* XOR A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->de.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xAB: /* XOR A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->de.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xAC: /* XOR A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->hl.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xAD: /* XOR A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->hl.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xAE: /* XOR A, (HL) */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h ^ m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
    case 0xAF: /* XOR A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h ^ cpu->af.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB0: /* OR A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->bc.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB1: /* OR A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->bc.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB2: /* OR A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->de.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB3: /* OR A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->de.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB4: /* OR A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->hl.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB5: /* OR A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->hl.b.l;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
    case 0xB6: /* OR A, (HL) */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h | m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
    case 0xB7: /* OR A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h | cpu->af.b.h;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* S, Z, P */
		cpu->af.b.h = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0xB8: /* CP A, B */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->bc.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.h) & 0x10) | /* H */
			(((cpu->bc.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xB9: /* CP A, C */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->bc.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->bc.b.l) & 0x10) | /* H */
			(((cpu->bc.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBA: /* CP A, D */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->de.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.h) & 0x10) | /* H */
			(((cpu->de.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBB: /* CP A, E */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->de.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->de.b.l) & 0x10) | /* H */
			(((cpu->de.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBC: /* CP A, H */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->hl.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.h) & 0x10) | /* H */
			(((cpu->hl.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBD: /* CP A, L */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->hl.b.l;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->hl.b.l) & 0x10) | /* H */
			(((cpu->hl.b.l ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xBE: /* CP A, (HL) */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->hl.w);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 7;
		break;
	}
	case 0xBF: /* CP A, A */
	{
		uint32 tmp;
		tmp = cpu->af.b.h - cpu->af.b.h;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ cpu->af.b.h) & 0x10) | /* H */
			(((cpu->af.b.h ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->cycles += 4;
		break;
	}
	case 0xC0: /* RET NZ */
	{
		if(cpu->af.b.l & 0x40)
		{
			/* Z = 1, no return */
			cpu->cycles += 5;
		}
		else	{
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		break;
	}
	case 0xC1: /* POP BC */
	{
		cpu->bc.b.l = cpu->mread(cpu->sp.w++);
		cpu->bc.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 10;
		break;
	}
	case 0xC2: /* JP NZ, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x40)	{
			/* Z = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xC3: /* JP nn */
	{
		reg_pair tmp;
		tmp.b.l = cpu->mread(cpu->pc.w++);
		tmp.b.h = cpu->mread(cpu->pc.w);
		cpu->pc.w = tmp.w;
		cpu->cycles += 10;
		break;
	}
	case 0xC4: /* CALL NZ, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x40)	{
			/* Z = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		break;
	}
	case 0xC5: /* PUSH BC */
	{
		cpu->mwrite(--cpu->sp.w, cpu->bc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->bc.b.l);
		cpu->cycles += 11;
		break;
	}
	case 0xC6: /* ADD A, n */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h + m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xC7: /* RST 0 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0000;
		cpu->cycles += 11;
		break;
	}
	case 0xC8: /* RET Z */
	{
		if(cpu->af.b.l & 0x40)
		{
			/* Z = 1, return */
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		else	{
			cpu->cycles += 5;
		}
		break;		
	}
	case 0xC9: /* RET */
	{
		cpu->pc.b.l = cpu->mread(cpu->sp.w++);
		cpu->pc.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 10;
		break;
	}
	case 0xCA: /* JP Z, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x40)	{
			/* Z = 1, jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */

		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xCB: /* CB-prefixed Opcodes */
	{
		uint8 op = cpu->mread(cpu->pc.w++);
		++cpu->ir.b.l;
		
#		include "z80_switch_CB.h"
		
		break;
	}
	case 0xCC: /* CALL Z, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x40)	{
			/* Z = 1,  jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		break;
	}
	case 0xCD: /* CALL nn */
	{
		reg_pair tmp;
		tmp.b.l = cpu->mread(cpu->pc.w++);
		tmp.b.h = cpu->mread(cpu->pc.w++);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = tmp.w;
		cpu->cycles += 17;
		break;
	}
	case 0xCE: /* ADC A, n */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h + m + (cpu->af.b.l & 0x01);
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h ^ 0x80) & (m ^ tmp) & 0x80) >> 5) | /* P */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xCF: /* RST 8 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0008;
		cpu->cycles += 11;
		break;
	}
	case 0xD0: /* RET NC */
	{
		if(cpu->af.b.l & 0x01)
		{
			/* C = 1, no return */
			cpu->cycles += 5;
		}
		else	{
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		break;
	}
	case 0xD1: /* POP DE */
	{
		cpu->de.b.l = cpu->mread(cpu->sp.w++);
		cpu->de.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 10;
		break;
	}
	case 0xD2: /* JP NC, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x01)	{
			/* C = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xD3: /* OUT (n), A */
	{
		uint8 tmp = cpu->mread(cpu->pc.w++);
		cpu->pwrite(tmp, cpu->af.b.h);
		cpu->cycles += 11;
		break;
	}
	case 0xD4: /* CALL NC, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x01)	{
			/* C = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		break;
	}
	case 0xD5: /* PUSH DE */
	{
		cpu->mwrite(--cpu->sp.w, cpu->de.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->de.b.l);
		cpu->cycles += 11;
		break;
	}
	case 0xD6: /* SUB A, n */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp & 0x100) >> 8); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xD7: /* RST 16 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0010;
		cpu->cycles += 11;
		break;
	}
	case 0xD8: /* RET C */
	{
		if(cpu->af.b.l & 0x01)
		{
			/* C = 1, return */
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		else	{
			cpu->cycles += 5;
		}
		break;
	}
	case 0xD9: /* EXX */
	{
		uint16 tmp = cpu->bc.w;
		cpu->bc.w = cpu->bcp.w;
		cpu->bcp.w = tmp;
		tmp = cpu->de.w;
		cpu->de.w = cpu->dep.w;
		cpu->dep.w = tmp;
		tmp = cpu->hl.w;
		cpu->hl.w = cpu->hlp.w;
		cpu->hlp.w = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0xDA: /* JP C, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x01)	{
			/* C = 1, jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xDB: /* IN A, (n) */
	{
		uint8 tmp = cpu->mread(cpu->pc.w++);
		cpu->af.b.h = cpu->pread(tmp);
		cpu->cycles += 11;
		break;
	}
	case 0xDC: /* CALL C, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x01)	{
			/* C = 1,  jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		break;
	}
	case 0xDD: /* DD-prefixed opcodes */
	{
		uint8 op = cpu->mread(cpu->pc.w++);
		cpu->ir.b.l += 2;
		
#		include "z80_switch_DD.h"
		
		break;
	}
	case 0xDE: /* SBC A, n */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h - (cpu->af.b.l & 0x01) - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* S, Z */
			((cpu->af.b.h ^ tmp ^ m) & 0x10) | /* H */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* P */
			0x02 | /* N */
			((tmp >> 8) & 0x01); /* C */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xDF: /* RST 24 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0018;
		cpu->cycles += 11;
		break;
	}
	case 0xE0: /* RET PO */
	{
		if(cpu->af.b.l & 0x04)
		{
			/* P = 1, no return */
			cpu->cycles += 5;
		}
		else	{
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		break;
	}
	case 0xE1: /* POP HL */
	{
		cpu->hl.b.l = cpu->mread(cpu->sp.w++);
		cpu->hl.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 10;
		break;
	}
	case 0xE2: /* JP PO, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x04)	{
			/* P = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xE3: /* EX (SP), HL */
	{
		uint16 tmp = cpu->hl.w;
		cpu->hl.b.l = cpu->mread(cpu->sp.w++);
		cpu->hl.b.h = cpu->mread(cpu->sp.w);
		cpu->mwrite(cpu->sp.w, (tmp >> 8) & 0xFF);
		cpu->mwrite(--cpu->sp.w, (tmp & 0xFF));
		cpu->cycles += 19;
		break;
	}
	case 0xE4: /* CALL PO, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x04)	{
			/* P = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		break;
	}
	case 0xE5: /* PUSH HL */
	{
		cpu->mwrite(--cpu->sp.w, cpu->hl.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->hl.b.l);
		cpu->cycles += 11;
		break;
	}
	case 0xE6: /* AND A, n */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h & m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF] | /* S, Z, P */
			0x10; /* H */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xE7: /* RST 32 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0020;
		cpu->cycles += 11;
		break;
	}
	case 0xE8: /* RET PE */
	{
		if(cpu->af.b.l & 0x04)
		{
			/* P = 1, return */
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		else	{
			cpu->cycles += 5;
		}
		break;
	}
	case 0xE9: /* JP (HL) (really JP HL) */
	{
		cpu->pc.w = cpu->hl.w;
		cpu->cycles += 4;
		break;
	}
	case 0xEA: /* JP PE, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x04)	{
			/* P = 1, jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xEB: /* EX DE, HL */
	{
		uint16 tmp = cpu->de.w;
		cpu->de.w = cpu->hl.w;
		cpu->hl.w = tmp;
		cpu->cycles += 4;
		break;
	}
	case 0xEC: /* CALL PE, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x04)	{
			/* P = 1,  jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		break;
	}
	case 0xED: /* ED-prefixed opcodes */
	{
		uint8 op = cpu->mread(cpu->pc.w++);
		++cpu->ir.b.l;

#		include "z80_switch_ED.h"

		break;
	}
	case 0xEE: /* XOR A, n */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h ^ m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;
	}
	case 0xEF: /* RST 40 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0028;
		cpu->cycles += 11;
		break;
	}
	case 0xF0: /* RET P */
	{
		if(cpu->af.b.l & 0x80)
		{
			/* S = 1, no return */
			cpu->cycles += 5;
		}
		else	{
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		break;
	}
	case 0xF1: /* POP AF */
	{
		cpu->af.b.l = cpu->mread(cpu->sp.w++);
		cpu->af.b.h = cpu->mread(cpu->sp.w++);
		cpu->cycles += 10;
		break;
	}
	case 0xF2: /* JP P, nn */
	{
		reg_pair tmp;

		if(cpu->af.b.l & 0x80)	{
			/* S = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xF3: /* DI */
	{
		cpu->iff1 = 0;
		cpu->iff2 = 0;
		cpu->cycles += 4;
		break;
	}
	case 0xF4: /* CALL P, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x80)	{
			/* S = 1, no jump */
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		else	{
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		break;
	}
	case 0xF5: /* PUSH AF */
	{
		cpu->mwrite(--cpu->sp.w, cpu->af.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->af.b.l);
		cpu->cycles += 11;
		break;
	}
	case 0xF6: /* OR A, n */
	{
		uint32 tmp;
        uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h | m;
		cpu->af.b.l = ZSPtable[tmp & 0xFF]; /* Set the Zero/Sign flags */
		cpu->af.b.h = tmp;
		cpu->cycles += 7;
		break;		
	}
	case 0xF7: /* RST 48 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0030;
		cpu->cycles += 11;
		break;
	}
	case 0xF8: /* RET M */
	{
		if(cpu->af.b.l & 0x80)
		{
			/* S = 1, return */
			cpu->pc.b.l = cpu->mread(cpu->sp.w++);
			cpu->pc.b.h = cpu->mread(cpu->sp.w++);
			cpu->cycles += 11;
		}
		else	{
			cpu->cycles += 5;
		}
		break;
	}
	case 0xF9: /* LD SP, HL */
	{
		cpu->sp.w = cpu->hl.w;
		cpu->cycles += 6;
		break;
	}
	case 0xFA: /* JP M, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x80)	{
			/* S = 1, jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w);
			cpu->pc.w = tmp.w;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			
		}
		cpu->cycles += 10; /* This doesn't seem to make sense, but meh */
		break;
	}
	case 0xFB: /* EI */
	{
		if(cpu->iff1 == 0)	{
			cpu->iff1 = 1;
			cpu->iff2 = 1;
			cpu->ei = 1;
			++cpu->ir.b.l;
		}
		else	{
			cpu->iff2 = 1;
		}
		cpu->cycles += 4;
		break;
	}
	case 0xFC: /* CALL M, nn */
	{
		reg_pair tmp;
		if(cpu->af.b.l & 0x80)	{
			/* S = 1,  jump */
			tmp.b.l = cpu->mread(cpu->pc.w++);
			tmp.b.h = cpu->mread(cpu->pc.w++);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
			cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
			cpu->pc.w = tmp.w;
			cpu->cycles += 17;
		}
		else	{
			cpu->pc.w += 2; /* Increment past the address */
			cpu->cycles += 10;
		}
		break;
	}
	case 0xFD: /* FD-prefixed opcodes */
	{
		uint8 op = cpu->mread(cpu->pc.w++);
		cpu->ir.b.l += 2;
		
#		include "z80_switch_FD.h"
		
		break;
	}
	case 0xFE: /* CP n */
	{
		uint32 tmp;
		uint8 m = cpu->mread(cpu->pc.w++);
		tmp = cpu->af.b.h - m;
		cpu->af.b.l = ZStable[tmp & 0xFF] | /* Set the Zero/Sign flags */
			((tmp >> 8) & 0x01) | /* Set the carry flag */
			(((m ^ cpu->af.b.h) & (cpu->af.b.h ^ tmp) & 0x80) >> 5) | /* Overflow */
			((m ^ cpu->af.b.h ^ tmp) & 0x10) | /* Half-carry */
			0x02; /* Subtraction */
		cpu->cycles += 7;
		break;
	}
	case 0xFF: /* RST 56 */
	{
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.h);
		cpu->mwrite(--cpu->sp.w, cpu->pc.b.l);
		cpu->pc.w = 0x0038;
		cpu->cycles += 11;
		break;
	}
	default:
	{
		printf("Error in emulation, bailing out, opcode = 0x%02X\n", inst);
		assert(0);
	}
}
