/*
	This file is part of CrabEmu.

	Copyright (C) 2005, 2006 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include "sms.h"
#include "smsvdp.h"
#include "smsmem.h"
#include "smsz80.h"
#include "sn76489.h"
#include "sound.h"

unsigned short sms_pad = 0xFFFF;
int sms_psg_enabled = 1;
int sms_initialized = 0;
sn76489_t psg;
int sms_console = CONSOLE_SMS;
int sms_region = SMS_REGION_EXPORT | SMS_VIDEO_NTSC;

static unsigned long psg_samples[262];

static const float Z80_CLOCK = 3579545.0f;
static const int PSG_DIVISOR = 16;
static const int FPS = 60;
static const int LINES_PER_FRAME = 262;
static const float CLOCKS_PER_SAMPLE = 5.073051303855f;

int sms_init()	{
	int i;
	float tmp;

	sms_mem_init();
	sms_vdp_init();
	sms_z80_init();
	sn76489_init(&psg);

	tmp = Z80_CLOCK / PSG_DIVISOR / FPS / LINES_PER_FRAME / CLOCKS_PER_SAMPLE;

	for(i = 0; i < 262; ++i)
		psg_samples[i] = (unsigned long) ((tmp * i) - (tmp * (i - 1)));

	sound_init();

    sms_initialized = 1;

	return 0;
}

int sms_reset() {
    if(sms_initialized == 0)
        return 0;

    sms_mem_shutdown();
    sms_mem_init();

    sms_z80_reset();

    return 0;
}

int sms_shutdown()	{
    sms_mem_shutdown();
	sms_vdp_shutdown();
	sms_z80_shutdown();
	sound_shutdown();

	return 0;
}

void sms_frame()	{
	int line;
	signed short buf[44100];

	for(line = 0; line < 262; ++line)	{
		sms_z80_run(227);
		sms_vdp_execute(line);

		if(sms_psg_enabled)	{
			sn76489_execute_samples(&psg, buf, psg_samples[line]);
			sound_update_buffer(buf, psg_samples[line] * 2);
		}
	}
}

void sms_button_pressed(int button)	{
	if(button == GG_START)	{
		if(sms_console == CONSOLE_GG)	{
			uint8 byte = sms_port_read(0);
			byte &= ~0x80;
			sms_port_write(0, byte);
		}
	}
	else	{
		sms_pad &= ~button;
	}
}

void sms_button_released(int button)	{
	if(button == GG_START)	{
		if(sms_console == CONSOLE_GG)	{
			uint8 byte = sms_port_read(0);
			byte |= 0x80;
			sms_port_write(0, byte);
		}
	}
	else	{
		sms_pad |= button;
	}
}
