/*
	This file is part of CrabEmu.

	Copyright (C) 2005 Lawrence Sebald

	CrabEmu is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CrabEmu is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CrabEmu; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#import <Cocoa/Cocoa.h>
#include "EmuOGLView.h"

#include <OpenGL/GL.h>
#include <time.h>
#include <sys/time.h>
#include "sms.h"
#include "smsvdp.h"

EmuOGLView *theview;
NSLock *memoryLock;
static GLuint glTexture;
static BOOL glInitted = NO;
extern sms_vdp_t smsvdp;
extern int sms_initialized;

static unsigned long long getMilliseconds(void);

static unsigned long long getMilliseconds(void)	{
	struct timeval tv;
	unsigned long long rv;

	gettimeofday(&tv, NULL);

	rv = tv.tv_sec * 1000;
	rv += tv.tv_usec / 1000;

	return rv;
}

void render_scene(void) {
    if(!glInitted) {
        if(!sms_initialized)
            return;

        glShadeModel(GL_SMOOTH);
        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        glClearDepth(1.0f);
        
        glViewport(0.0f, 0.0f, 256.0f, 192.0f);
        
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(0.0f, 256.0f, 192.0f, 0.0f, 1.0f, 0.0f);
        
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        
        glGenTextures(1, &glTexture);
        glBindTexture(GL_TEXTURE_2D, glTexture);
        glTexImage2D(GL_TEXTURE_2D, 0, 4, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, smsvdp.framebuffer);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        
        glEnable(GL_TEXTURE_2D);
        
        glInitted = YES;
    }
    
    glClear(GL_COLOR_BUFFER_BIT);
    glLoadIdentity();
    
    glBindTexture(GL_TEXTURE_2D, glTexture);
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 256, 256, GL_RGBA, GL_UNSIGNED_BYTE, smsvdp.framebuffer);
    
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(0.0f, 192.0f / 256.0f);
    glVertex2f(0.0f, 192.0f);
    glTexCoord2f(0.0f, 0.0f);
    glVertex2f(0.0f, 0.0f);
    glTexCoord2f(1.0f, 192.0f / 256.0f);
    glVertex2f(256.0f, 192.0f);
    glTexCoord2f(1.0f, 0.0f);
    glVertex2f(256.0f, 0.0f);
    glEnd();
    
    glFlush();
}

void emu_loop(void)	{
#ifndef TIMING_TEST
	unsigned long long ms = getMilliseconds();
#else
    time_t thetime = time(NULL);
    unsigned long frames = 0;
#endif

	for(;;)	{
        [memoryLock lock];

		sms_frame();

        [theview display];

        [memoryLock unlock];

#ifndef TIMING_TEST
		while(ms + 16 > getMilliseconds()) ;
		ms = getMilliseconds();
#else
        ++frames;

        if(thetime + 20 <= time(NULL))  {
            printf("%d frames in %d seconds = %f FPS\n", frames, time(NULL) - thetime, (float) frames / (time(NULL) - thetime));
            thetime = time(NULL);
            frames = 0;
        }
#endif
	}
}

void gui_set_viewport(int w, int h)  {
    [theview resizeWindowWithViewWidth: (float) w height: h];
}

int main(int argc, char *argv[])	{
    int retval;

    memoryLock = [[NSLock alloc] init];

    retval = NSApplicationMain(argc,  (const char **) argv);

    [memoryLock release];
}
