/*
    This file is part of CrabEmu.

    Copyright (C) 2005, 2006, 2007 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

    CrabEmu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CrabEmu; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef CRABZ80_H
#define CRABZ80_H

#ifndef CRABEMU_TYPEDEFS
#define CRABEMU_TYPEDEFS

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned long uint32;

typedef signed char int8;
typedef signed short int16;
typedef signed long int32;

#endif

typedef union   {
    struct  {
#ifdef __BIG_ENDIAN__
        uint8 h;
        uint8 l;
#else
        uint8 l;
        uint8 h;
#endif
    } b;
    uint16 w;
} CrabZ80_reg_t;

typedef struct CrabZ80_struct   {
    CrabZ80_reg_t af;
    CrabZ80_reg_t bc;
    CrabZ80_reg_t de;
    CrabZ80_reg_t hl;
    CrabZ80_reg_t ix;
    CrabZ80_reg_t iy;
    CrabZ80_reg_t pc;
    CrabZ80_reg_t sp;
    CrabZ80_reg_t ir;
    CrabZ80_reg_t afp;
    CrabZ80_reg_t bcp;
    CrabZ80_reg_t dep;
    CrabZ80_reg_t hlp;

    CrabZ80_reg_t *offset;

    uint8 internal_reg;
    uint8 iff1;
    uint8 iff2;
    uint8 im;

    uint8 halt;
    uint8 ei;
    uint8 r_top;

    uint32 cycles;
    uint32 cycles_in;

    uint8 (*pread)(uint16 port);
    uint8 (*mread)(uint16 addr);

    void (*pwrite)(uint16 port, uint8 data);
    void (*mwrite)(uint16 addr, uint8 data);
} CrabZ80_t;

/* Flag definitions */
#define CRABZ80_CF      0
#define CRABZ80_NF      1
#define CRABZ80_PF      2
#define CRABZ80_XF      3
#define CRABZ80_HF      4
#define CRABZ80_YF      5
#define CRABZ80_ZF      6
#define CRABZ80_SF      7

/* Flag setting macros */
#define CRABZ80_SET_FLAG(z80, n)    (z80)->af.b.l |= (1 << (n))
#define CRABZ80_CLEAR_FLAG(z80, n)  (z80)->af.b.l &= (~(1 << (n)))
#define CRABZ80_GET_FLAG(z80, n)    ((z80)->af.b.l >> (n)) & 1

/* Function definitions */
void CrabZ80_init(CrabZ80_t *cpu);
void CrabZ80_reset(CrabZ80_t *cpu);

void CrabZ80_pulse_nmi(CrabZ80_t *cpu);
void CrabZ80_pulse_irq(CrabZ80_t *cpu);

uint32 CrabZ80_execute(CrabZ80_t *cpu, uint32 cycles);

void CrabZ80_release_cycles();

void CrabZ80_set_portread(CrabZ80_t *cpu, uint8 (*pread)(uint16 port));
void CrabZ80_set_memread(CrabZ80_t *cpu, uint8 (*mread)(uint16 addr));
void CrabZ80_set_portwrite(CrabZ80_t *cpu,
                           void (*pwrite)(uint16 port, uint8 data));
void CrabZ80_set_memwrite(CrabZ80_t *cpu,
                          void (*mwrite)(uint16 addr, uint8 data));

#endif /* !CRABZ80_H */
