/*
    This file is part of CrabEmu.

    Copyright (C) 2005, 2006, 2007 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

    CrabEmu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CrabEmu; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef CRABZ80_H
#define CRABZ80_H

#ifdef _arch_dreamcast
#include <arch/types.h>
#define CRABEMU_TYPEDEFS
#endif

#ifndef CRABEMU_TYPEDEFS
#define CRABEMU_TYPEDEFS

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned long uint32;

typedef signed char int8;
typedef signed short int16;
typedef signed long int32;

#endif

typedef union   {
    struct  {
#ifdef __BIG_ENDIAN__
        uint8 h;
        uint8 l;
#else
        uint8 l;
        uint8 h;
#endif
    } b;
    uint16 w;
} CrabZ80_reg_t;

typedef struct CrabZ80_struct   {
    CrabZ80_reg_t af;  // 0
    CrabZ80_reg_t bc;  // 2
    CrabZ80_reg_t de;  // 4
    CrabZ80_reg_t hl;  // 6
    CrabZ80_reg_t ix;  // 8
    CrabZ80_reg_t iy;  // 10
    CrabZ80_reg_t pc;  // 12
    CrabZ80_reg_t sp;  // 14
    CrabZ80_reg_t ir;  // 16
    CrabZ80_reg_t afp; // 18
    CrabZ80_reg_t bcp; // 20
    CrabZ80_reg_t dep; // 22
    CrabZ80_reg_t hlp; // 24

    CrabZ80_reg_t *offset; // 28

    uint8 internal_reg; // 32
    uint8 iff1; // 33
    uint8 iff2; // 34
    uint8 im;   // 35

    uint8 halt; // 36
    uint8 ei;   // 37
    uint8 irq_pending; // 38
    uint8 r_top;// 39

    uint32 cycles; // 40
    uint32 cycles_in; // 44

    uint8 (*pread)(uint16 port); // 48
    uint8 (*mread)(uint16 addr); // 52

    void (*pwrite)(uint16 port, uint8 data); // 56
    void (*mwrite)(uint16 addr, uint8 data); // 60
} CrabZ80_t;

/* Flag definitions */
#define CRABZ80_CF      0
#define CRABZ80_NF      1
#define CRABZ80_PF      2
#define CRABZ80_XF      3
#define CRABZ80_HF      4
#define CRABZ80_YF      5
#define CRABZ80_ZF      6
#define CRABZ80_SF      7

/* Flag setting macros */
#define CRABZ80_SET_FLAG(z80, n)    (z80)->af.b.l |= (1 << (n))
#define CRABZ80_CLEAR_FLAG(z80, n)  (z80)->af.b.l &= (~(1 << (n)))
#define CRABZ80_GET_FLAG(z80, n)    ((z80)->af.b.l >> (n)) & 1

/* Function definitions */
void CrabZ80_init(CrabZ80_t *cpu);
void CrabZ80_reset(CrabZ80_t *cpu);

void CrabZ80_pulse_nmi(CrabZ80_t *cpu);
void CrabZ80_assert_irq(CrabZ80_t *cpu);
void CrabZ80_clear_irq(CrabZ80_t *cpu);

uint32 CrabZ80_execute(CrabZ80_t *cpu, uint32 cycles);

void CrabZ80_release_cycles();

void CrabZ80_set_portread(CrabZ80_t *cpu, uint8 (*pread)(uint16 port));
void CrabZ80_set_memread(CrabZ80_t *cpu, uint8 (*mread)(uint16 addr));
void CrabZ80_set_portwrite(CrabZ80_t *cpu,
                           void (*pwrite)(uint16 port, uint8 data));
void CrabZ80_set_memwrite(CrabZ80_t *cpu,
                          void (*mwrite)(uint16 addr, uint8 data));

#endif /* !CRABZ80_H */
