/*
    This file is part of CrabEmu.

    Copyright (C) 2008 Lawrence Sebald

    CrabEmu is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 
    as published by the Free Software Foundation.

    CrabEmu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CrabEmu; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "terebi.h"
#include "smsmem.h"

static unsigned char terebi_x;
static unsigned char terebi_y;
static int terebi_flags;

extern unsigned char *read_map[256];
extern unsigned char *write_map[256];

/* This function is... well, not pretty. Apparently the Terebi Oekaki isn't all
   that accurate in actuality. Many thanks to Omar Cornut of MEKA fame for
   figuring out how to actually calculate what should be sent to the game.
   (The odd math in here is adapted from MEKA's TVOekaki_Update function) */
void terebi_update(int x, int y, int pressed)   {
    x -= 5;
    y -= 4;

    if(x < 0)
        x = 0;
    else if(x > 255)
        x = 255;

    if(y < 0)
        y = 0;
    else if(y > 191)
        y = 191;

    y += 32;
    
    terebi_x = (unsigned char)x;
    terebi_y = (unsigned char)y;

    if(pressed) {
        terebi_flags |= TEREBI_OEKAKI_PRESSED;
    }
    else    {
        terebi_flags &= ~TEREBI_OEKAKI_PRESSED;
    }
}

unsigned char terebi_mread(unsigned short addr) {
    if(addr == 0x8000)  {
        return (terebi_flags & TEREBI_OEKAKI_PRESSED) ? 0 : 1;
    }
    else if(addr == 0xA000)     {
        if((terebi_flags & TEREBI_OEKAKI_PRESSED)) {
            return (terebi_flags & TEREBI_OEKAKI_AXIS_Y) ? terebi_x : terebi_y;
        }
        else    {
            return 0;
        }
    }

    return read_map[addr >> 8][addr & 0xFF];
}

void terebi_mwrite(unsigned short addr, unsigned char data) {
    if(addr == 0x6000)  {
        if(data & 0x01) {
            terebi_flags |= TEREBI_OEKAKI_AXIS_Y;
        }
        else    {
            terebi_flags &= ~TEREBI_OEKAKI_AXIS_Y;
        }
    }

    write_map[addr >> 8][addr & 0xFF] = data;
}

unsigned short terebi_mread16(unsigned short addr)  {
    unsigned short rv;
    int top, bot;

    if(addr == 0x8000)  {
        return (unsigned short)(terebi_flags & TEREBI_OEKAKI_PRESSED);
    }
    else if(addr == 0xA000)     {
        if((terebi_flags & TEREBI_OEKAKI_PRESSED)) {
            return (terebi_flags & TEREBI_OEKAKI_AXIS_Y) ? terebi_y : terebi_x;
        }
        else    {
            return 0;
        }
    }

    top = addr >> 8;
    bot = addr & 0xFF;

    rv = read_map[top][bot++];

    if(bot <= 0xFF) {
        return rv | (read_map[top][bot] << 8);
    }
    else    {
        return rv | (read_map[(unsigned char)(top + 1)][0] << 8);
    }
}

void terebi_mwrite16(unsigned short addr, unsigned short data)  {
    int top, bot;

    if(addr == 0x6000)  {
        if(data & 0x01) {
            terebi_flags |= TEREBI_OEKAKI_AXIS_Y;
        }
        else    {
            terebi_flags &= ~TEREBI_OEKAKI_AXIS_Y;
        }
    }

    top = addr >> 8;
    bot = addr & 0xFF;

    write_map[top][bot++] = (unsigned char)data;
    
    if(bot <= 0xFF)
        write_map[top][bot] = (unsigned char)(data >> 8);
    else
        write_map[(unsigned char)(top + 1)][0] = (unsigned char)(data >> 8);
}
